@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2002   The R Development Core Team.
+ *  Copyright (C) 1997-2003   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,17 +15,17 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 #ifndef PRINT_H_
 #define PRINT_H_
 
-#include "R_ext/PrtUtil.h"
+#include <R_ext/PrtUtil.h>
 #include "Defn.h"
 
-#ifndef R_NO_REMAP
 #define formatFactor        Rf_formatFactor
+#define formatRaw           Rf_formatRaw
 #define formatString        Rf_formatString
 #define EncodeFactor        Rf_EncodeFactor
 #define EncodeElement       Rf_EncodeElement
@@ -33,13 +33,13 @@
 #define printMatrix         Rf_printMatrix
 #define printNamedVector    Rf_printNamedVector
 #define printVector         Rf_printVector
-#endif
 
 typedef struct {
     int width;
     int na_width;
     int na_width_noquote;
     int digits;
+    int scipen;
     int gap;
     int quote;
     int right;
@@ -50,11 +50,12 @@ extern R_print_par_t R_print;
 
 /* Computation of printing formats */
 void formatFactor(int *, int, int *, SEXP, int);
+void formatRaw(Rbyte *, int, int *);
 void formatString(SEXP*, int, int*, int);
 
 /* Formating of values */
 char *EncodeFactor(int, int, int, SEXP);
-char *EncodeElement(SEXP, int, int);
+char *EncodeElement(SEXP, int, int, char);
 
 /* Printing */
 void MatrixColumnLabel(SEXP, int, int);
@@ -62,9 +63,9 @@ void RightMatrixColumnLabel(SEXP, int, int);
 void LeftMatrixColumnLabel(SEXP, int, int);
 void MatrixRowLabel(SEXP, int, int, int);
 
-/* In Defn.h (and MUST be there):
+/* In Rinternals.h (and MUST be there):
    CustomPrintValue,  PrintValue, PrintValueRec */
-void printArray(SEXP, SEXP, int, SEXP);
+void printArray(SEXP, SEXP, int, int, SEXP);
 void printMatrix(SEXP, int, SEXP, int, int, SEXP, SEXP, char*, char*);
 void printNamedVector(SEXP, SEXP, int, char*);
 void printVector(SEXP, int, int);
