@@ -25,7 +25,7 @@ function(file, header = FALSE, sep = "", quote = "\"'", dec = ".",
          nrows = -1, skip = 0,
          check.names = TRUE, fill = !blank.lines.skip,
          strip.white = FALSE, blank.lines.skip = TRUE,
-         comment.char = "#")
+         comment.char = "#", allowEscapes = FALSE, flush = FALSE)
 {
     if(is.character(file)) {
         file <- file(file, "r")
@@ -58,14 +58,14 @@ function(file, header = FALSE, sep = "", quote = "\"'", dec = ".",
     if(all(nchar(lines) == 0)) stop("empty beginning of file")
     if(nlines < n0lines && file == 0)  {# stdin() has reached EOF
         pushBack(c(lines, lines, ""), file)
-        on.exit(.Internal(clearPushback(stdin())))
+        on.exit(.Internal(clearPushBack(stdin())))
     } else
         pushBack(c(lines, lines), file)
     first <- scan(file, what = "", sep = sep, quote = quote,
                   nlines = 1, quiet = TRUE, skip = 0,
                   strip.white = TRUE,
                   blank.lines.skip = blank.lines.skip,
-                  comment.char = comment.char)
+                  comment.char = comment.char, allowEscapes = allowEscapes)
     col1 <- if(missing(col.names)) length(first) else length(col.names)
     col <- numeric(nlines - 1)
     if (nlines > 1)
@@ -75,7 +75,8 @@ function(file, header = FALSE, sep = "", quote = "\"'", dec = ".",
                                   nlines = 1, quiet = TRUE, skip = 0,
                                   strip.white = strip.white,
                                   blank.lines.skip = blank.lines.skip,
-                                  comment.char = comment.char))
+                                  comment.char = comment.char,
+                                  allowEscapes = allowEscapes))
     cols <- max(col1, col)
 
     ##	basic column counting and header determination;
@@ -139,7 +140,8 @@ function(file, header = FALSE, sep = "", quote = "\"'", dec = ".",
 		 na.strings = na.strings, quiet = TRUE, fill = fill,
                  strip.white = strip.white,
                  blank.lines.skip = blank.lines.skip, multi.line = FALSE,
-                 comment.char = comment.char)
+                 comment.char = comment.char, allowEscapes = allowEscapes,
+                 flush = flush)
 
     nlines <- length(data[[ which(keep)[1] ]])
 
@@ -184,7 +186,7 @@ function(file, header = FALSE, sep = "", quote = "\"'", dec = ".",
             else if (colClasses[i] == "factor") as.factor(data[[i]])
             else if (colClasses[i] == "Date") as.Date(data[[i]])
             else if (colClasses[i] == "POSIXct") as.POSIXct(data[[i]])
-            else as(data[[i]], colClasses[i])
+            else methods::as(data[[i]], colClasses[i])
     }
 
     ##	now determine row names
