@@ -1,6 +1,6 @@
 .saveRDS <-
 function(object, file = "", ascii = FALSE, version = NULL,
-         compress = FALSE, refhook = baseenv())
+         compress = FALSE, refhook = NULL)
 {
     if(is.character(file)) {
         if(file == "") stop("'file' must be non-empty string")
@@ -31,21 +31,25 @@ function(file, refhook = NULL)
     .Internal(unserializeFromConn(con, refhook))
 }
 
-serialize <- function(object, connection, ascii = FALSE, refhook = NULL) {
-    if (! is.null(connection)) {
+serialize <- function(object, connection, ascii = FALSE, refhook = NULL)
+{
+    if (!is.null(connection)) {
         if (!inherits(connection, "connection"))
             stop("'connection' must be a connection")
         if (missing(ascii)) ascii <- summary(connection)$text == "text"
     }
-    if (! ascii && inherits(connection, "sockconn"))
+    if (!ascii && inherits(connection, "sockconn"))
         .Call("R_serializeb", object, connection, refhook, PACKAGE="base")
     else
         .Call("R_serialize", object, connection, ascii, refhook,
               PACKAGE="base")
 }
 
-unserialize <- function(connection, refhook = NULL) {
-    if (! is.character(connection) && !inherits(connection, "connection"))
+unserialize <- function(connection, refhook = NULL)
+{
+    if (typeof(connection) != "raw" &&
+        !is.character(connection) &&
+        !inherits(connection, "connection"))
         stop("'connection' must be a connection")
     .Call("R_unserialize", connection, refhook, PACKAGE="base")
 }
