@@ -3,6 +3,7 @@
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *  Copyright (C) 1998--2003  Guido Masarotto and Brian Ripley
  *  Copyright (C) 2004        The R Foundation
+ *  Copyright (C) 2004-5      The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,10 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-
-/* <MBCS> Use of strchr on display type is fine.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 /*--- Device Driver for Windows; this file started from
@@ -39,7 +37,8 @@
 #include "graphapp/stdimg.h"
 #include "console.h"
 #include "rui.h"
-#include "windows.h"
+#define WIN32_LEAN_AND_MEAN 1
+#include <windows.h>
 #include "devWindows.h"
 #include "grDevices.h"
 #ifdef ENABLE_NLS
@@ -55,7 +54,7 @@ Rboolean GADeviceDriver(NewDevDesc *dd, char *display, double width,
 			double height, double pointsize,
 			Rboolean recording, int resize, int bg, int canvas,
 			double gamma, int xpos, int ypos, Rboolean buffered,
-			SEXP psenv);
+			SEXP psenv, Rboolean restoreConsole);
 
 
 /* a colour used to represent the background on png if transparent
@@ -245,7 +244,7 @@ static void PrivateCopyDevice(NewDevDesc *dd, NewDevDesc *ndd, char *name)
     gadesc *xd = (gadesc *) dd->deviceSpecific;
     gsetcursor(xd->gawin, WatchCursor);
     gsetVar(install(".Device"),
-	    mkString(name), R_NilValue);
+	    mkString(name), R_BaseEnv);
     gdd = GEcreateDevDesc(ndd);
     addDevice((DevDesc*) gdd);
     GEcopyDisplayList(devNumber((DevDesc*) dd));
@@ -256,7 +255,7 @@ static void PrivateCopyDevice(NewDevDesc *dd, NewDevDesc *ndd, char *name)
     show(xd->gawin);
 }
 
-static void SaveAsWin(NewDevDesc *dd, char *display)
+static void SaveAsWin(NewDevDesc *dd, char *display, Rboolean restoreConsole)
 {
     NewDevDesc *ndd = (NewDevDesc *) calloc(1, sizeof(NewDevDesc));
     GEDevDesc* gdd = (GEDevDesc*) GetDevice(devNumber((DevDesc*) dd));
@@ -278,7 +277,7 @@ static void SaveAsWin(NewDevDesc *dd, char *display)
 					GE_INCHES, gdd),
 		       ((gadesc*) dd->deviceSpecific)->basefontsize,
 		       0, 1, White, White, 1, NA_INTEGER, NA_INTEGER, FALSE,
-		       R_GlobalEnv))
+		       R_GlobalEnv, restoreConsole))
         PrivateCopyDevice(dd, ndd, display);
 }
 
@@ -335,7 +334,8 @@ static void SaveAsPostscript(NewDevDesc *dd, char *fn)
 	    }
 	}
     }
-    if (PSDeviceDriver(ndd, fn, paper, family, afmpaths, encoding, bg, fg,
+    if (PSDeviceDriver(ndd, fn, paper, family, afmpaths, encoding,
+                       bg, fg,
 		       fromDeviceWidth(toDeviceWidth(1.0, GE_NDC, gdd),
 				       GE_INCHES, gdd),
 		       fromDeviceHeight(toDeviceHeight(-1.0, GE_NDC, gdd),
@@ -353,7 +353,7 @@ static void SaveAsPDF(NewDevDesc *dd, char *fn)
     NewDevDesc *ndd = (NewDevDesc *) calloc(1, sizeof(NewDevDesc));
     GEDevDesc* gdd = (GEDevDesc*) GetDevice(devNumber((DevDesc*) dd));
     gadesc *xd = (gadesc *) dd->deviceSpecific;
-    char family[256], encoding[256], bg[256], fg[256];
+    char family[256], encoding[256], bg[256], fg[256], **afmpaths = NULL;
 
     if (!ndd) {
 	R_ShowMessage(_("Not enough memory to copy graphics window"));
@@ -392,7 +392,8 @@ static void SaveAsPDF(NewDevDesc *dd, char *fn)
 	    }
 	}
     }
-    if (PDFDeviceDriver(ndd, fn, "special", family, encoding, bg, fg,
+    if (PDFDeviceDriver(ndd, fn, "special", family, afmpaths, encoding,
+                        bg, fg,
 			fromDeviceWidth(toDeviceWidth(1.0, GE_NDC, gdd),
 					GE_INCHES, gdd),
 			fromDeviceHeight(toDeviceHeight(-1.0, GE_NDC, gdd),
@@ -510,7 +511,7 @@ static int SetBaseFont(gadesc *xd)
     xd->usefixed = FALSE;
     xd->fontfamily[0] = '\0';
     xd->font = gnewfont(xd->gawin, fontname[0], fontstyle[0],
-			MulDiv(xd->fontsize, xd->wanteddpi, xd->truedpi), 0.0);
+			xd->fontsize, 0.0);
     if (!xd->font) {
 	xd->usefixed= TRUE;
 	xd->font = xd->fixedfont = FixedFont;
@@ -535,7 +536,7 @@ static char *SaveFontSpec(SEXP sxp, int offset)
 /*
  * Take the fontfamily from a gcontext (which is device-independent)
  * and convert it into a Windows-specific font description using
- * the Windows font database (see src/library/graphics/R/unix/windows.R)
+ * the Windows font database (see src/library/grDevices/R/windows/windows.R)
  *
  * IF gcontext fontfamily is empty ("")
  * OR IF can't find gcontext fontfamily in font database
@@ -549,7 +550,7 @@ static char* translateFontFamily(char* family) {
 
     PROTECT(graphicsNS = R_FindNamespace(ScalarString(mkChar("grDevices"))));
     PROTECT_WITH_INDEX(windowsenv = findVar(install(".Windowsenv"),
-					   graphicsNS), &xpi);
+					    graphicsNS), &xpi);
     if(TYPEOF(windowsenv) == PROMSXP)
 	REPROTECT(windowsenv = eval(windowsenv, graphicsNS), xpi);
     PROTECT(fontdb = findVar(install(".Windows.Fonts"), windowsenv));
@@ -557,7 +558,7 @@ static char* translateFontFamily(char* family) {
     nfonts = LENGTH(fontdb);
     if (strlen(family) > 0) {
 	int found = 0;
-	for (i=0; i<nfonts && !found; i++) {
+	for (i = 0; i < nfonts && !found; i++) {
 	    char* fontFamily = CHAR(STRING_ELT(fontnames, i));
 	    if (strcmp(family, fontFamily) == 0) {
 		found = 1;
@@ -590,7 +591,6 @@ static void SetFont(char *family, int face, int size, double rot,
 	face = 1;
     if (size < SMALLEST) size = SMALLEST;
     if (size > LARGEST) size = LARGEST;
-    size = MulDiv(size, xd->wanteddpi, xd->truedpi);
     if (!xd->usefixed &&
 	(size != xd->fontsize || face != xd->fontface ||
 	 rot != xd->fontangle || strcmp(family, xd->fontfamily))) {
@@ -755,9 +755,12 @@ static void HelpExpose(window w, rect r)
 
 	if (xd->resize) {
 	    GA_Resize(dd);
-	    xd->replaying = TRUE;
-	    GEplayDisplayList(gdd);
-	    xd->replaying = FALSE;
+	    /* avoid trying to replay list if there has been no drawing */
+	    if(gdd->dirty) {
+		xd->replaying = TRUE;
+		GEplayDisplayList(gdd);
+		xd->replaying = FALSE;
+	    }
 	    R_ProcessEvents();
 	} else
 	    SHOW;
@@ -775,16 +778,18 @@ static void HelpMouseClick(window w, int button, point pt)
 	    return;
 	if (button & LeftButton) {
 	    int useBeep = xd->locator && asLogical(GetOption(install("locatorBell"),
-					      R_NilValue));
+					      R_BaseEnv));
 	    if(useBeep) gabeep();
 	    xd->clicked = 1;
 	    xd->px = pt.x;
 	    xd->py = pt.y;
 	} else
 	    xd->clicked = 2;
-	if (dd->gettingEvent)  
+	if (dd->gettingEvent) { 
 	    xd->eventResult = doMouseEvent(xd->eventRho, dd, meMouseDown, 
 	    			button, pt.x, pt.y);
+            if (xd->buffered) SHOW;	
+        }
     }
 }
 
@@ -795,9 +800,11 @@ static void HelpMouseMove(window w, int button, point pt)
 	NewDevDesc *dd = (NewDevDesc *) getdata(w);
 	gadesc *xd = (gadesc *) dd->deviceSpecific;
 
-	if (dd->gettingEvent)  
+	if (dd->gettingEvent) {
 	    xd->eventResult = doMouseEvent(xd->eventRho, dd, meMouseMove, 
 	    			button, pt.x, pt.y);
+	    if (xd->buffered) SHOW;	    			
+	}
     }
 }
 
@@ -808,9 +815,11 @@ static void HelpMouseUp(window w, int button, point pt)
 	NewDevDesc *dd = (NewDevDesc *) getdata(w);
 	gadesc *xd = (gadesc *) dd->deviceSpecific;
 
-	if (dd->gettingEvent)  
+	if (dd->gettingEvent) { 
 	    xd->eventResult = doMouseEvent(xd->eventRho, dd, meMouseUp,
 	    			button, pt.x, pt.y);
+	    if (xd->buffered) SHOW;	
+	}
     }
 }
 
@@ -901,14 +910,14 @@ static void menuwm(control m)
 	return;
     }
     sprintf(display, "win.metafile:%s", fn);
-    SaveAsWin(dd, display);
+    SaveAsWin(dd, display, TRUE);
 }
 
 
 static void menuclpwm(control m)
 {
     NewDevDesc *dd = (NewDevDesc *) getdata(m);
-    SaveAsWin(dd, "win.metafile");
+    SaveAsWin(dd, "win.metafile", TRUE);
 }
 
 static void menuclpbm(control m)
@@ -933,7 +942,7 @@ static void menustayontop(control m)
 static void menuprint(control m)
 {
     NewDevDesc *dd = (NewDevDesc *) getdata(m);
-    SaveAsWin(dd, "win.print:");
+    SaveAsWin(dd, "win.print:", TRUE);
 }
 
 static void menuclose(control m)
@@ -1259,8 +1268,10 @@ static void CHelpKeyIn(control w, int key)
     
     if (dd->gettingEvent) {
     	keyname = getKeyName(key);
-    	if (keyname > knUNKNOWN) 
+    	if (keyname > knUNKNOWN) {
     	    xd->eventResult = doKeybd(xd->eventRho, dd, keyname, NULL);
+	    if (xd->buffered) SHOW;	
+	}
     } else {
 	if (xd->replaying) return;
 	switch (key) {
@@ -1280,7 +1291,7 @@ static void CHelpKeyIn(control w, int key)
     }
 }
 
-extern Rboolean UserBreak;
+__declspec(dllimport) extern int UserBreak;
 
 static void NHelpKeyIn(control w, int key)
 {
@@ -1298,6 +1309,7 @@ static void NHelpKeyIn(control w, int key)
 	    keyname[1] = '\0';
 	}
 	xd->eventResult = doKeybd(xd->eventRho, dd, knUNKNOWN, keyname);
+	if (xd->buffered) SHOW;	
     } else {
 	if (xd->replaying) return;
 	switch (key) {
@@ -1695,9 +1707,13 @@ static Rboolean GA_Open(NewDevDesc *dd, gadesc *xd, char *dsp,
 	    warning(_("Unable to load Rbitmap.dll"));
 	    return FALSE;
 	}
+
+	if (w < 20 && h < 20)
+	    warning(_("'width=%d, height=%d' are unlikely values in pixels"),
+		    (int)w, (int) h);
 	/*
 	  Observe that given actual graphapp implementation 256 is
-	  irrelevant,i.e., depth of the bitmap is the one of graphic card
+	  irrelevant,i.e., depth of the bitmap is that of graphic card
 	  if required depth > 1
 	*/
 	if ((xd->gawin = newbitmap(w, h, 256)) == NULL) {
@@ -1725,6 +1741,9 @@ static Rboolean GA_Open(NewDevDesc *dd, gadesc *xd, char *dsp,
 	*p = ':' ;
 	if(strlen(p+1) >= 512) error(_("filename too long in jpeg() call"));
 	strcpy(xd->filename, p+1);
+	if (w < 20 && h < 20)
+	    warning(_("'width=%d, height=%d' are unlikely values in pixels"),
+		    (int)w, (int) h);
 	if((xd->gawin = newbitmap(w, h, 256)) == NULL) {
 	    warning(_("Unable to allocate bitmap"));
 	    return FALSE;
@@ -1822,7 +1841,11 @@ static double GA_StrWidth(char *str,
 	/********************************************************/
 
 	/* Character Metric Information */
-	/* Passing c == 0 gets font information */
+	/* Passing c == 0 gets font information.
+	   In a mbcslocale for a non-symbol font 
+	   we pass a Unicode point, otherwise an 8-bit char, and
+	   we don't care which for a 7-bit char.
+	 */
 
 static void GA_MetricInfo(int c,
 			  R_GE_gcontext *gc,
@@ -1834,8 +1857,8 @@ static void GA_MetricInfo(int c,
     gadesc *xd = (gadesc *) dd->deviceSpecific;
 
     SetFont(gc->fontfamily, gc->fontface, size, 0.0, dd);
-#ifdef SUPPORT_UTF8
-    if(gc->fontface != 5)
+#ifdef SUPPORT_MBCS
+    if(mbcslocale && gc->fontface != 5 && c > 127)
 	gwcharmetric(xd->gawin, xd->font, c, &a, &d, &w);
     else 
 #endif
@@ -1845,12 +1868,11 @@ static void GA_MetricInfo(int c,
     if(c == 32) {
 	*ascent  = 0.0;
 	*descent = 0.0;
-	*width   = (double) w;
     } else {
 	*ascent  = (double) a;
 	*descent = (double) d;
-	*width   = (double) w;
     }
+    *width   = (double) w;
 }
 
 	/********************************************************/
@@ -1884,16 +1906,10 @@ static void GA_Size(double *left, double *right,
 		     double *bottom, double *top,
 		     NewDevDesc *dd)
 {
-    gadesc *xd = (gadesc *) dd->deviceSpecific;
-
-    int   iw, ih;
-
-    iw = xd->windowWidth;
-    ih = xd->windowHeight;
-    *left = 0.0;
-    *top = 0.0;
-    *right = iw;
-    *bottom = ih;
+    *left = dd->left;
+    *top = dd->top;
+    *right = dd->right;
+    *bottom = dd->bottom;
 }
 
 static void GA_Resize(NewDevDesc *dd)
@@ -2004,6 +2020,8 @@ static void GA_NewPage(R_GE_gcontext *gc,
 	    del(xd->gawin);
 	    snprintf(buf, 600, xd->filename, xd->npage);
 	    xd->gawin = newmetafile(buf, xd->w, xd->h);
+	    if(!xd->gawin)
+		error(_("metafile '%s' could not be created"), buf);
 	}
     }
     if ((xd->kind == PNG || xd->kind == JPEG || xd->kind == BMP)
@@ -2085,7 +2103,7 @@ static void GA_Close(NewDevDesc *dd)
 	deleteGraphMenus(devNumber((DevDesc*) dd) + 1);
     } else if ((xd->kind == PNG) || (xd->kind == JPEG) || (xd->kind == BMP)) {
       SaveAsBitmap(dd, xd->res_dpi);
-    }
+    } 
     del(xd->font);
     del(xd->gawin);
 /*
@@ -2383,13 +2401,10 @@ static void GA_Text(double x, double y, char *str,
     SetFont(gc->fontfamily, gc->fontface, size, rot, dd);
     SetColor(gc->col, gc->gamma, dd);
     if (R_OPAQUE(gc->col)) {
-#ifdef SUPPORT_UTF8
-	if(gc->fontface != 5) {
+	if(mbcslocale && gc->fontface != 5) {
 	    /* These macros need to be wrapped in braces */
-	    DRAW(gwdrawstr(_d, xd->font, xd->fgcolor, pt(x, y), str, hadj));
-	} else 
-#endif
-	{
+	    DRAW(gwdrawstr1(_d, xd->font, xd->fgcolor, pt(x, y), str, hadj));
+	} else {
 	    DRAW(gdrawstr1(_d, xd->font, xd->fgcolor, pt(x, y), str, hadj));
 	}
     }
@@ -2403,9 +2418,29 @@ static void GA_Text(double x, double y, char *str,
 	/* not all devices will do anything (e.g., postscript)	*/
 	/********************************************************/
 
+static void donelocator(void *data)
+{
+    gadesc *xd;
+    xd = (gadesc *)data;
+    addto(xd->gawin);
+    gchangemenubar(xd->mbar);
+    if (xd->stoploc) {
+      hide(xd->stoploc);
+      show(xd->gawin);
+    }
+    gsetcursor(xd->gawin, ArrowCursor);
+    gchangepopup(xd->gawin, xd->grpopup);
+    addto(xd->gawin);
+    setstatus(_("R Graphics"));
+    xd->locator = FALSE;
+}
+
+static void GA_onExit(NewDevDesc *dd);
+
 static Rboolean GA_Locator(double *x, double *y, NewDevDesc *dd)
 {
     gadesc *xd = (gadesc *) dd->deviceSpecific;
+    RCNTXT cntxt;
 
     if (xd->kind != SCREEN)
 	return FALSE;
@@ -2421,22 +2456,29 @@ static Rboolean GA_Locator(double *x, double *y, NewDevDesc *dd)
     gchangepopup(xd->gawin, xd->locpopup);
     gsetcursor(xd->gawin, CrossCursor);
     setstatus(G_("Locator is active"));
+
+    /* set up a context which will clean up if there's an error */
+    begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_NilValue, R_NilValue,
+	         R_NilValue, R_NilValue);
+    cntxt.cend = &donelocator;
+    cntxt.cenddata = xd;
+    xd->cntxt = &cntxt;
+    
+    /* and an exit handler in case the window gets closed */
+    dd->onExit = GA_onExit;
+    
     while (!xd->clicked) {
-	if(xd->buffered) SHOW;
+	SH;
         WaitMessage();
 	R_ProcessEvents();
     }
-    addto(xd->gawin);
-    gchangemenubar(xd->mbar);
-    if (xd->stoploc) {
-      hide(xd->stoploc);
-      show(xd->gawin);
-    }
-    gsetcursor(xd->gawin, ArrowCursor);
-    gchangepopup(xd->gawin, xd->grpopup);
-    addto(xd->gawin);
-    setstatus(_("R Graphics"));
-    xd->locator = FALSE;
+    
+    dd->onExit = NULL;
+    xd->cntxt = NULL;
+    
+    endcontext(&cntxt);
+    donelocator((void *)xd);
+    
     if (xd->clicked == 1) {
 	*x = xd->px;
 	*y = xd->py;
@@ -2504,7 +2546,7 @@ Rboolean GADeviceDriver(NewDevDesc *dd, char *display, double width,
 			double height, double pointsize,
 			Rboolean recording, int resize, int bg, int canvas,
 			double gamma, int xpos, int ypos, Rboolean buffered,
-			SEXP psenv)
+			SEXP psenv, Rboolean restoreConsole)
 {
     /* if need to bail out with some sort of "error" then */
     /* must free(dd) */
@@ -2566,6 +2608,7 @@ Rboolean GADeviceDriver(NewDevDesc *dd, char *display, double width,
     dd->hold = GA_Hold;
     dd->metricInfo = GA_MetricInfo;
     xd->newFrameConfirm = GA_NewFrameConfirm;
+    xd->cntxt = NULL;
 
     /* set graphics parameters that must be set by device driver */
     /* Window Dimensions in Pixels */
@@ -2628,7 +2671,7 @@ Rboolean GADeviceDriver(NewDevDesc *dd, char *display, double width,
     xd->buffered = buffered;
     xd->psenv = psenv;
     {
-	SEXP timeouts = GetOption(install("windowsTimeouts"), R_NilValue);
+	SEXP timeouts = GetOption(install("windowsTimeouts"), R_BaseEnv);
 	if(isInteger(timeouts)){
 	    xd->timeafter = INTEGER(timeouts)[0];
 	    xd->timesince = INTEGER(timeouts)[1];
@@ -2640,7 +2683,7 @@ Rboolean GADeviceDriver(NewDevDesc *dd, char *display, double width,
     }
     xd->newFrameConfirm = GA_NewFrameConfirm;
     dd->displayListOn = (xd->kind == SCREEN);
-    if (RConsole && (xd->kind!=SCREEN)) show(RConsole);
+    if (RConsole && restoreConsole) show(RConsole);
     return TRUE;
 }
 
@@ -2650,6 +2693,7 @@ SEXP savePlot(SEXP args)
     char *fn, *tp, display[550];
     int device;
     NewDevDesc* dd;
+    Rboolean restoreConsole;
 
     args = CDR(args); /* skip entry point name */
     device = asInteger(CAR(args));
@@ -2666,7 +2710,8 @@ SEXP savePlot(SEXP args)
     if (!isString(type) || LENGTH(type) != 1)
 	error(_("invalid type argument in savePlot"));
     tp = CHAR(STRING_ELT(type, 0));
-
+    restoreConsole = asLogical(CADDDR(args));
+    
     if(!strcmp(tp, "png")) {
 	SaveAsPng(dd, fn);
     } else if (!strcmp(tp,"bmp")) {
@@ -2674,14 +2719,14 @@ SEXP savePlot(SEXP args)
     } else if(!strcmp(tp, "jpeg") || !strcmp(tp,"jpg")) {
       /*Default quality suggested in libjpeg*/
         SaveAsJpeg(dd, 75, fn);
-    } else if (!strcmp(tp, "wmf")) {
+    } else if (!strcmp(tp, "wmf") || !strcmp(tp, "emf")) {
 	if(strlen(fn) > 512) {
 	    askok(G_("file path selected is too long: only 512 bytes are allowed"));
 	    return R_NilValue;
 	}
 	sprintf(display, "win.metafile:%s", fn);
-	SaveAsWin(dd, display);
-    } else if (!strcmp(tp, "ps")) {
+	SaveAsWin(dd, display, restoreConsole);
+    } else if (!strcmp(tp, "ps") || !strcmp(tp, "eps")) {
 	SaveAsPostscript(dd, fn);
     } else if (!strcmp(tp, "pdf")) {
 	SaveAsPDF(dd, fn);
@@ -2692,7 +2737,7 @@ SEXP savePlot(SEXP args)
 
 
 /* Rbitmap  */
-#define BITMAP_DLL_NAME "\\BIN\\RBITMAP.DLL\0"
+#define BITMAP_DLL_NAME "\\modules\\Rbitmap.dll\0"
 typedef int (*R_SaveAsBitmap)();
 static R_SaveAsBitmap R_SaveAsPng, R_SaveAsJpeg, R_SaveAsBmp;
 
@@ -2878,6 +2923,7 @@ SEXP devga(SEXP args)
     char *display, *vmax;
     double height, width, ps, xpinch, ypinch, gamma;
     int recording = 0, resize = 1, bg, canvas, xpos, ypos, buffered;
+    Rboolean restoreConsole;
     SEXP sc, psenv;
 
     vmax = vmaxget();
@@ -2925,6 +2971,8 @@ SEXP devga(SEXP args)
     if (!isString(sc) && !isInteger(sc) && !isLogical(sc) && !isReal(sc))
 	error(_("invalid value of 'bg' in devWindows"));
     bg = RGBpar(sc, 0);
+    args = CDR(args);
+    restoreConsole = asLogical(CAR(args));
     
     R_CheckDeviceAvailable();
     BEGIN_SUSPEND_INTERRUPTS {
@@ -2940,12 +2988,13 @@ SEXP devga(SEXP args)
 	GAsetunits(xpinch, ypinch);
 	if (!GADeviceDriver(dev, display, width, height, ps, 
 			    (Rboolean)recording, resize, bg, canvas, gamma,
-			    xpos, ypos, (Rboolean)buffered, psenv)) {
+			    xpos, ypos, (Rboolean)buffered, psenv, 
+			    restoreConsole)) {
 	    free(dev);
 	    error(_("unable to start device devWindows"));
 	}
 	gsetVar(install(".Device"),
-		mkString(display[0] ? display : "windows"), R_NilValue);
+		mkString(display[0] ? display : "windows"), R_BaseEnv);
 	dd = GEcreateDevDesc(dev);
 	addDevice((DevDesc*) dd);
 	GEinitDisplayList(dd);
@@ -2963,6 +3012,9 @@ static void GA_onExit(NewDevDesc *dd)
     dd->gettingEvent = FALSE;
     xd->eventRho = NULL;
     
+    if (xd->cntxt) endcontext(xd->cntxt);
+    if (xd->locator) donelocator((void *)xd);
+    
     addto(xd->gawin);
     gchangemenubar(xd->mbar);
     gchangepopup(xd->gawin, xd->grpopup);
@@ -2995,7 +3047,7 @@ static Rboolean GA_NewFrameConfirm()
     settext(xd->gawin, G_("Click or hit ENTER for next page"));
     dd->dev->onExit = GA_onExit;  /* install callback for cleanup */
     while (!xd->clicked && !xd->enterkey) {
-	if(xd->buffered) SHOW;
+	SH;
         WaitMessage();
 	R_ProcessEvents(); /* May not return if user interrupts */
     }
@@ -3028,7 +3080,7 @@ static SEXP GA_getEvent(SEXP eventRho, char* prompt)
     xd->eventResult = NULL;
     dd->dev->onExit = GA_onExit;  /* install callback for cleanup */
     while (!xd->eventResult || xd->eventResult == R_NilValue) {
-	if(xd->buffered) SHOW;
+	SH;
         WaitMessage();
 	R_ProcessEvents(); /* May not return if user interrupts */
     }
