@@ -13,13 +13,16 @@ add1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
 #     data <- model.frame(update(object, newform)) # remove NAs
 #     object <- update(object, data = data)
     ns <- length(scope)
-    ans <- matrix(nrow = ns + 1, ncol = 2)
-    dimnames(ans) <- list(c("<none>", scope), c("df", "AIC"))
-    ans[1, ] <- extractAIC(object, scale, k = k, ...)
+    ans <- matrix(nrow = ns + 1, ncol = 2,
+                  dimnames = list(c("<none>", scope), c("df", "AIC")))
+    ans[1,  ] <- extractAIC(object, scale, k = k, ...)
     n0 <- length(object$residuals)
     for(i in seq(ns)) {
 	tt <- scope[i]
-	if(trace > 1) cat("trying +", tt, "\n")
+	if(trace > 1) {
+            cat("trying +", tt, "\n")
+	    flush.console()
+        }
 	nfit <- update(object, as.formula(paste("~ . +", tt)),
                        evaluate = FALSE)
         nfit <- eval.parent(nfit)
@@ -81,31 +84,44 @@ add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     if(is.null(x)) {
 	fc <- object$call
 	fc$formula <- Terms
-	fob <- list(call = fc)
+	## model.frame.lm looks at the terms part for the environment
+	fob <- list(call = fc, terms = Terms)
 	class(fob) <- oldClass(object)
 	m <- model.frame(fob, xlev = object$xlevels)
 	x <- model.matrix(Terms, m, contrasts = object$contrasts)
+        offset <- model.offset(m)
+        wt <- model.weights(m)
         oldn <- length(y)
         y <- model.response(m, "numeric")
         newn <- length(y)
         if(newn < oldn)
             warning(gettextf("using the %d/%d rows from a combined fit",
                              newn, oldn), domain = NA)
+    } else {
+        ## need to get offset and weights from somewhere
+        wt <- object$weights
+        offset <- object$offset
     }
     n <- nrow(x)
     Terms <- attr(Terms, "term.labels")
     asgn <- attr(x, "assign")
     ousex <- match(asgn, match(oTerms, Terms), 0) > 0
     if(int) ousex[1] <- TRUE
-    iswt <- !is.null(wt <- object$weights)
+    iswt <- !is.null(wt)
     X <- x[, ousex, drop = FALSE]
-    z <- if(iswt) lm.wfit(X, y, wt) else lm.fit(X, y)
+    z <- if(iswt) lm.wfit(X, y, wt, offset=offset)
+    else lm.fit(X, y, offset=offset)
     dfs[1] <- z$rank
     RSS[1] <- deviance.lm(z)
+    ## workaround for PR#7842. terms.formula may have flipped interactions
+    sTerms <- sapply(strsplit(Terms, ":", fixed=TRUE),
+                     function(x) paste(sort(x), collapse=":"))
     for(tt in scope) {
-	usex <- match(asgn, match(tt, Terms), 0) > 0
+        stt <- paste(sort(strsplit(tt, ":")[[1]]), collapse=":")
+	usex <- match(asgn, match(stt, sTerms), 0) > 0
 	X <- x[, usex|ousex, drop = FALSE]
-	z <- if(iswt) lm.wfit(X, y, wt) else lm.fit(X, y)
+	z <- if(iswt) lm.wfit(X, y, wt, offset=offset)
+        else lm.fit(X, y, offset=offset)
 	dfs[tt] <- z$rank
 	RSS[tt] <- deviance.lm(z)
     }
@@ -169,22 +185,33 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     new.form <- update.formula(object, add.rhs)
     Terms <- terms(new.form)
     y <- object$y
-    wt <- object$prior.weights
     if(is.null(x)) {
 	fc <- object$call
 	fc$formula <- Terms
-	fob <- list(call = fc)
+	## model.frame.glm looks at the terms part for the environment
+	fob <- list(call = fc, terms = Terms)
 	class(fob) <- oldClass(object)
 	m <- model.frame(fob, xlev = object$xlevels)
+        offset <- model.offset(m)
+        wt <- model.weights(m)
 	x <- model.matrix(Terms, m, contrasts = object$contrasts)
         oldn <- length(y)
         y <- model.response(m, "numeric")
-        ## binomial case has adjusted y.
-        if(NCOL(y) == 2) y <- y[, 1]/(y[, 1] + y[,2])
+        ## binomial case has adjusted y and weights
+        if(NCOL(y) == 2) {
+            n <- y[, 1] + y[, 2]
+            y <- ifelse(n == 0, 0, y[, 1]/n)
+            if(is.null(wt)) wt <- rep.int(1, length(y))
+            wt <- wt * n
+        }
         newn <- length(y)
         if(newn < oldn)
             warning(gettextf("using the %d/%d rows from a combined fit",
                              newn, oldn), domain = NA)
+    } else {
+        ## need to get offset and weights from somewhere
+        wt <- object$prior.weights
+        offset <- object$offset
     }
     n <- nrow(x)
     if(is.null(wt)) wt <- rep.int(1, n)
@@ -193,14 +220,18 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     ousex <- match(asgn, match(oTerms, Terms), 0) > 0
     if(int) ousex[1] <- TRUE
     X <- x[, ousex, drop = FALSE]
-    z <-  glm.fit(X, y, wt, offset=object$offset,
+    z <-  glm.fit(X, y, wt, offset=offset,
                   family=object$family, control=object$control)
     dfs[1] <- z$rank
     dev[1] <- z$deviance
+    ## workaround for PR#7842. terms.formula may have flipped interactions
+    sTerms <- sapply(strsplit(Terms, ":", fixed=TRUE),
+                     function(x) paste(sort(x), collapse=":"))
     for(tt in scope) {
-	usex <- match(asgn, match(tt, Terms), 0) > 0
+        stt <- paste(sort(strsplit(tt, ":")[[1]]), collapse=":")
+	usex <- match(asgn, match(stt, sTerms), 0) > 0
 	X <- x[, usex|ousex, drop = FALSE]
-	z <-  glm.fit(X, y, wt, offset=object$offset,
+	z <-  glm.fit(X, y, wt, offset=offset,
 		      family=object$family, control=object$control)
 	dfs[tt] <- z$rank
 	dev[tt] <- z$deviance
@@ -263,14 +294,17 @@ drop1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
 #    data <- model.frame(object) # remove NAs
 #    object <- update(object, data = data)
     ns <- length(scope)
-    ans <- matrix(nrow = ns + 1, ncol = 2)
-    dimnames(ans) <- list(c("<none>", scope), c("df", "AIC"))
-    ans[1, ] <- extractAIC(object, scale, k = k, ...)
+    ans <- matrix(nrow = ns + 1, ncol = 2,
+                  dimnames =  list(c("<none>", scope), c("df", "AIC")))
+    ans[1,  ] <- extractAIC(object, scale, k = k, ...)
     n0 <- length(object$residuals)
     for(i in seq(ns)) {
 	tt <- scope[i]
-	if(trace > 1) cat("trying -", tt, "\n")
-	nfit <- update(object, as.formula(paste("~ . -", tt)),
+	if(trace > 1) {
+            cat("trying -", tt, "\n")
+	    flush.console()
+        }
+        nfit <- update(object, as.formula(paste("~ . -", tt)),
                        evaluate = FALSE)
         nfit <- eval.parent(nfit)
 	ans[i+1, ] <- extractAIC(nfit, scale, k = k, ...)
@@ -301,6 +335,7 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
 		     test=c("none", "Chisq", "F"), k = 2, ...)
 {
     x <- model.matrix(object)
+    offset <- model.offset(model.frame(object))
     iswt <- !is.null(wt <- object$weights)
     n <- nrow(x)
     asgn <- attr(x, "assign")
@@ -324,10 +359,11 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
 	ii <- seq(along=asgn)[asgn == ndrop[i]]
 	if(all.cols) jj <- setdiff(seq(ncol(x)), ii)
 	else jj <- setdiff(na.coef, ii)
-	z <- if(iswt) lm.wfit(x[, jj, drop = FALSE], y, wt)
-	else lm.fit(x[, jj, drop = FALSE], y)
+	z <- if(iswt) lm.wfit(x[, jj, drop = FALSE], y, wt, offset=offset)
+	else lm.fit(x[, jj, drop = FALSE], y, offset=offset)
 	dfs[i] <- z$rank
-	RSS[i] <- deviance.lm(z)
+        oldClass(z) <- "lm" # needed as deviance.lm calls residuals.lm
+	RSS[i] <- deviance(z)
     }
     scope <- c("<none>", scope)
     dfs <- c(object$rank, dfs)
@@ -438,7 +474,7 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
         aod[, "Pr(Chi)"] <- dev
     } else if(test == "F") {
         if(fam == "binomial" || fam == "poisson")
-            warning(gettextf("F test assumes quasi%s family", fam),
+            warning(gettextf("F test assumes 'quasi%s' family", fam),
                     domain = NA)
 	dev <- aod$Deviance
 	rms <- dev[1]/rdf
@@ -487,8 +523,17 @@ factor.scope <- function(factor, scope)
 	facs <- factor
 	if(length(drop)) {
 	    nmfac <- colnames(factor)
-	    where <- match(nmdrop, nmfac, 0)
-	    if(any(!where)) stop("lower scope is not included in model")
+            ## workaround as in PR#7842.
+            ## terms.formula may have flipped interactions
+            nmfac0 <- sapply(strsplit(nmfac, ":", fixed=TRUE),
+                             function(x) paste(sort(x), collapse=":"))
+            nmdrop0 <- sapply(strsplit(nmdrop, ":", fixed=TRUE),
+                             function(x) paste(sort(x), collapse=":"))
+	    where <- match(nmdrop0, nmfac0, 0)
+	    if(any(!where))
+                stop(gettextf("lower scope has term(s) %s not included in model",
+                              paste(sQuote(nmdrop[where==0]), collapse=", ")),
+                     domain = NA)
 	    facs <- factor[, -where, drop = FALSE]
 	    nmdrop <- nmfac[-where]
 	} else nmdrop <- colnames(factor)
@@ -506,8 +551,17 @@ factor.scope <- function(factor, scope)
 	nmfac <- colnames(factor)
 	nmadd <- colnames(add)
 	if(!is.null(nmfac)) {
-	    where <- match(nmfac, nmadd, 0)
-	    if(any(!where)) stop("upper scope does not include model")
+            ## workaround as in PR#7842.
+            ## terms.formula may have flipped interactions
+            nmfac0 <- sapply(strsplit(nmfac, ":", fixed=TRUE),
+                             function(x) paste(sort(x), collapse=":"))
+            nmadd0 <- sapply(strsplit(nmadd, ":", fixed=TRUE),
+                             function(x) paste(sort(x), collapse=":"))
+	    where <- match(nmfac0, nmadd0, 0)
+	    if(any(!where))
+                stop(gettextf("upper scope does not include model term(s) %s",
+                              paste(sQuote(nmfac[where==0]), collapse=", ")),
+                     domain = NA)
 	    nmadd <- nmadd[-where]
 	    add <- add[, -where, drop = FALSE]
 	}
