@@ -69,10 +69,14 @@ ftable.formula <- function(formula, data = NULL, subset, na.action, ...)
         stop("'formula' missing or incorrect")
     if(length(formula) != 3)
         stop("'formula' must have both left and right hand sides")
-    if(any(attr(terms(formula), "order") > 1))
+    ## need to cope with '.' in formula
+    tt <- if(is.data.frame(data)) terms(formula, data=data)
+    else terms(formula, allowDotAsName=TRUE)
+    if(any(attr(tt, "order") > 1))
         stop("interactions are not allowed")
-    rvars <- attr(terms(formula[-2]), "term.labels")
-    cvars <- attr(terms(formula[-3]), "term.labels")
+    ## here we do NOT want '.' expanded
+    rvars <- attr(terms(formula[-2], allowDotAsName=TRUE), "term.labels")
+    cvars <- attr(terms(formula[-3], allowDotAsName=TRUE), "term.labels")
     rhs.has.dot <- any(rvars == ".")
     lhs.has.dot <- any(cvars == ".")
     if(lhs.has.dot && rhs.has.dot)
@@ -147,7 +151,7 @@ as.table.ftable <- function(x, ...)
     x
 }
 
-write.ftable <- function(x, file = "", quote = TRUE,
+write.ftable <- function(x, file = "", quote = TRUE, append = FALSE,
                          digits = getOption("digits"))
 {
     if(!inherits(x, "ftable"))
@@ -187,7 +191,8 @@ write.ftable <- function(x, file = "", quote = TRUE,
                   format(unclass(x), digits = digits))
     x <- cbind(apply(LABS, 2, format, justify = "left"),
                apply(DATA, 2, format, justify = "right"))
-    cat(t(x), file = file, sep = c(rep(" ", ncol(x) - 1), "\n"))
+    cat(t(x), file = file, append = append,
+        sep = c(rep(" ", ncol(x) - 1), "\n"))
     invisible(ox)
 }
 
