@@ -1,7 +1,7 @@
 smooth.spline <-
-  function(x, y = NULL, w = NULL, df, spar = NULL, cv = FALSE,
-	   all.knots = FALSE, nknots = NULL,
-           df.offset = 0, penalty = 1, control.spar = list())
+    function(x, y = NULL, w = NULL, df, spar = NULL, cv = FALSE,
+             all.knots = FALSE, nknots = NULL, keep.data = TRUE,
+             df.offset = 0, penalty = 1, control.spar = list())
 {
     sknotl <- function(x, nk = NULL)
     {
@@ -97,7 +97,7 @@ smooth.spline <-
     iparms <- as.integer(c(icrit,ispar, contr.sp$maxit))
     names(iparms) <- c("icrit", "ispar", "iter")
 
-    fit <- .Fortran("qsbart",		# code in ../src/qsbart.f
+    fit <- .Fortran(R_qsbart,		# code in ../src/qsbart.f
 		    as.double(penalty),
 		    as.double(dofoff),
 		    x = as.double(xbar),
@@ -119,7 +119,7 @@ smooth.spline <-
 		    ld4  = as.integer(4),
 		    ldnk = as.integer(1),
 		    ier = integer(1),
-		    DUP = FALSE, PACKAGE="stats"
+		    DUP = FALSE
 		    )[c("coef","ty","lev","spar","parms","crit","iparms","ier")]
 
     lev <- fit$lev
@@ -152,6 +152,7 @@ smooth.spline <-
     class(fit.object) <- "smooth.spline.fit"
     ## parms :  c(low = , high = , tol = , eps = )
     object <- list(x = ux, y = fit$ty, w = wbar, yin = ybar,
+                   data = if(keep.data) list(x = x, y = y, w = w),
 		   lev = lev, cv.crit = cv.crit, pen.crit = pen.crit,
                    crit = fit$crit,
                    df = df, spar = fit$spar,
@@ -162,6 +163,36 @@ smooth.spline <-
     object
 }
 
+fitted.smooth.spline <- function(object, ...) {
+    if(!is.list(dat <- object$data))
+        stop("need result of smooth.spline(*, keep.data=TRUE)")
+    ## note that object$x == unique(sort(object$data$x))
+    object$y[match(dat$x, object$x)]
+}
+
+residuals.smooth.spline <-
+    function (object, type = c("working", "response", "deviance",
+                      "pearson", "partial"), ...)
+{
+    type <- match.arg(type)
+    if(!is.list(dat <- object$data))
+        stop("need result of smooth.spline(*, keep.data=TRUE)")
+    r <- dat$y - object$y[match(dat$x, object$x)]
+    ## this rest is `as' residuals.lm() :
+    res <- switch(type,
+                  working = ,
+                  response = r,
+                  deviance = ,
+                  pearson = if (is.null(dat$w)) r else r * sqrt(dat$w),
+                  partial = r)
+    res <- naresid(object$na.action, res)
+    if (type == "partial")
+        stop('type = "partial" is not yet implemented')
+        ## res <- res + predict(object, type = "terms")
+    res
+}
+
+
 print.smooth.spline <- function(x, digits = getOption("digits"), ...)
 {
     if(!is.null(cl <- x$call)) {
@@ -205,7 +236,7 @@ predict.smooth.spline.fit <- function(object, x, deriv = 0, ...)
     n <- sum(interp) # number of xs in [0,1]
     y <- xs
     if(any(interp))
-	y[interp] <- .Fortran("bvalus",
+	y[interp] <- .Fortran(R_bvalus,
 			      n	  = as.integer(n),
 			      knot= as.double(object$knot),
 			      coef= as.double(object$coef),
@@ -213,7 +244,7 @@ predict.smooth.spline.fit <- function(object, x, deriv = 0, ...)
 			      x	  = as.double(xs[interp]),
 			      s	  = double(n),
 			      order= as.integer(deriv),
-			      DUP = FALSE, PACKAGE="stats")$s
+			      DUP = FALSE)$s
     if(any(extrap)) {
 	xrange <- c(object$min, object$min + object$range)
 	if(deriv == 0) {
@@ -260,8 +291,8 @@ supsmu <-
     leno <- length(ord)
     if(diff <- n - leno)
 	warning(diff, " observation(s) with NAs, NaNs and/or Infs deleted")
-    .Fortran("setsmu", PACKAGE = "stats")
-    smo <- .Fortran("supsmu",
+    .Fortran(R_setsmu)
+    smo <- .Fortran(R_supsmu,
 		    as.integer(leno),
 		    as.double(xo),
 		    as.double(y[ord]),
@@ -270,8 +301,7 @@ supsmu <-
 		    as.double(span),
 		    as.double(bass),
 		    smo=double(leno),
-		    double(n*7), double(1),
-		    PACKAGE = "stats")$smo
+		    double(n*7), double(1))$smo
     ## eliminate duplicate xsort values and corresponding smoothed values
     dupx <- duplicated(xo)
     list(x = xo[!dupx], y = smo[!dupx])
