@@ -1,11 +1,15 @@
 install.packages <-
-    function(pkgs, lib, repos = CRAN,
+    function(pkgs, lib, repos = getOption("repos"),
              contriburl = contrib.url(repos, type),
-             CRAN = getOption("repos"),
              method, available = NULL, destdir = NULL,
              installWithVers = FALSE, dependencies = FALSE,
-             type = getOption("pkgType"))
+             type = getOption("pkgType"), configure.args = character(0),
+             clean = FALSE)
 {
+
+    if (is.logical(clean) && clean)
+        clean <- "--clean"
+
     explode_bundles <- function(a)
     {
         contains <- .find_bundles(a, FALSE)
@@ -24,6 +28,37 @@ install.packages <-
     	pkgs
     }
 
+
+      # Compute the configuration arguments for a given package.
+      # If configure.args is an unnamed character vector, use that.
+      # If it is named, match the pkg name to the names of the character vector
+      # and if we get a match, use that element.
+      # Similarly, configure.args is a list(), match pkg to the names pkg and
+      # use that element, collapsing it into a single string.
+
+    getConfigureArgs <-  function(pkg)
+    {
+        ## Since the pkg argument can be the name of a file rather than
+        ## a regular package name, we have to clean that up.
+        pkg <- gsub("_\\.(zip|tar\\.gz)", "",
+                    gsub(.standard_regexps()$valid_package_version, "", basename(pkg)))
+
+        if(length(pkgs) == 1 && length(names(configure.args)))
+            return(paste("--configure-args=",
+                         shQuote(paste(configure.args, collapse = " ")),
+                         sep = ""))
+
+        if (length(configure.args) && length(names(configure.args))
+              && pkg %in% names(configure.args))
+            config <- paste("--configure-args=",
+                            shQuote(paste(configure.args[[ pkg ]], collapse = " ")),
+                            sep = "")
+        else
+            config <- character(0)
+
+        config
+    }
+
     if(missing(pkgs) || !length(pkgs)) {
         if(.Platform$OS.type == "windows" || .Platform$GUI == "AQUA") {
             if(is.null(available))
@@ -83,17 +118,22 @@ install.packages <-
         if(type == "win.binary")
             stop("cannot install Windows binary packages on this plaform")
 
-        if(!file.exists(file.path(R.home(), "bin", "INSTALL")))
+        if(!file.exists(file.path(R.home("bin"), "INSTALL")))
             stop("This version of R is not set up to install source packages\nIf it was installed from an RPM, you may need the R-devel RPM")
     }
 
     if(is.null(repos) & missing(contriburl)) {
-        update <- cbind(pkgs, lib) # for side-effect of recycling to same length
-        cmd0 <- paste(file.path(R.home(),"bin","R"), "CMD INSTALL")
+        update <- cbind(path.expand(pkgs), lib) # for side-effect of recycling to same length
+        cmd0 <- paste(file.path(R.home("bin"),"R"), "CMD INSTALL")
         if (installWithVers)
             cmd0 <- paste(cmd0, "--with-package-versions")
+        if (is.character(clean))
+            cmd0 <- paste(cmd0, clean)
+
         for(i in 1:nrow(update)) {
+
             cmd <- paste(cmd0, "-l", shQuote(update[i, 2]),
+                          getConfigureArgs(update[i, 1]),
                          shQuote(update[i, 1]))
             if(system(cmd) > 0)
                 warning(gettextf(
@@ -130,15 +170,12 @@ install.packages <-
     if(depends) { # check for dependencies, recursively
         p0 <- p1 <- unique(pkgs) # this is ok, as 1 lib only
         have <- .packages(all.available = TRUE)
+        not_avail <- character(0)
 	repeat {
 	    if(any(miss <- ! p1 %in% row.names(available))) {
-		cat(sprintf(ngettext(sum(miss),
-				     "dependency %s is not available",
-				     "dependencies %s are not available"),
-		    paste(sQuote(p1[miss]), collapse=", ")), "\n\n", sep ="")
-		flush.console()
+                not_avail <- c(not_avail, p1[miss])
+                p1 <- p1[!miss]
 	    }
-	    p1 <- p1[!miss]
 	    deps <- as.vector(available[p1, dependencies])
 	    deps <- .clean_up_dependencies(deps, available)
 	    if(!length(deps)) break
@@ -147,6 +184,15 @@ install.packages <-
 	    pkgs <- c(toadd, pkgs)
 	    p1 <- toadd
 	}
+        if(length(not_avail)) {
+            cat(sprintf(ngettext(sum(miss),
+                                 "dependency '%s' is not available",
+                                 "dependencies '%s' are not available"),
+                        paste(sQuote(not_avail), collapse=", ")),
+                "\n\n", sep ="")
+            flush.console()
+        }
+
         for(bundle in names(bundles))
             pkgs[ pkgs %in% bundles[[bundle]] ] <- bundle
         pkgs <- unique(pkgs)
@@ -161,14 +207,14 @@ install.packages <-
         }
     }
 
-    pkgs <- unique(pkgs) # in case ask for more than one from a bundle
-    foundpkgs <- download.packages(pkgs, destdir = tmpd, available = available,
+    foundpkgs <- download.packages(unique(pkgs), destdir = tmpd,
+                                   available = available,
                                    contriburl = contriburl, method = method,
                                    type = "source")
 
     ## at this point pkgs may contain duplicates,
     ## the same pkg in different libs
-    if(!is.null(foundpkgs)) {
+    if(length(foundpkgs)) {
         update <- unique(cbind(pkgs, lib))
         colnames(update) <- c("Package", "LibPath")
         found <- pkgs %in% foundpkgs[, 1]
@@ -181,11 +227,19 @@ install.packages <-
             ## can't use update[p0, ] due to possible multiple matches
             update <- update[sort.list(match(pkgs, p0)), ]
         }
-        cmd0 <- paste(file.path(R.home(),"bin","R"), "CMD INSTALL")
+        cmd0 <- file.path(R.home("bin"),"R")
+	## setting R_ARCH obviates the need for this
+        ## if(nchar(r_arch <- .Platform$r_arch))
+        ##     cmd0 <- paste(cmd0, "--arch", r_arch)
+        cmd0 <- paste(cmd0, "CMD INSTALL")
         if (installWithVers)
             cmd0 <- paste(cmd0, "--with-package-versions")
+        if (is.character(clean))
+            cmd0 <- paste(cmd0, clean)
+
         for(i in 1:nrow(update)) {
-            cmd <- paste(cmd0, "-l", shQuote(update[i, 2]), update[i, 3])
+            cmd <- paste(cmd0, "-l", shQuote(update[i, 2]),
+                          getConfigureArgs(update[i, 3]), update[i, 3])
             status <- system(cmd)
             if(status > 0)
                 warning(gettextf(
