@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 /* <UTF8>
@@ -161,8 +161,7 @@ do_unzip(char *zipname, char *dest, int nfiles, char **files,
     return err;
 }
 
-SEXP 
-do_int_unzip(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_int_unzip(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP  fn, ans, names = R_NilValue;
     char  zipname[PATH_MAX], *topics[500], dest[PATH_MAX], *p;
@@ -179,13 +178,13 @@ do_int_unzip(SEXP call, SEXP op, SEXP args, SEXP env)
     ntopics = length(fn);
     if (ntopics > 0) {
 	if (!isString(fn) || ntopics > 500)
-	    errorcall(call, _("invalid 'topics' argument"));
+	    errorcall(call, _("invalid '%s' argument"), "topics");
 	for (i = 0; i < ntopics; i++)
 	    topics[i] = CHAR(STRING_ELT(fn, i));
     }
     args = CDR(args);
     if (!isString(CAR(args)) || LENGTH(CAR(args)) != 1)
-	errorcall(call, _("invalid 'destination' argument"));
+	errorcall(call, _("invalid '%s' argument"), "destination");
     p = R_ExpandFileName(CHAR(STRING_ELT(CAR(args), 0)));
     if (strlen(p) > PATH_MAX - 1)
 	errorcall(call, _("'destination' is too long"));
