@@ -14,9 +14,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
  */
 
+/* <UTF8> char here is either ASCII or handled as a whole */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -31,7 +33,7 @@ static R_InternetRoutines routines, *ptr = &routines;
 
 
 /*
-SEXP do_download(SEXP call, SEXP op, SEXP args, SEXP env);
+SEXP attribute_hidden do_download(SEXP call, SEXP op, SEXP args, SEXP env);
 Rconnection R_newurl(char *description, char *mode);
 Rconnection R_newsock(char *host, int port, int server, char *mode);
 
@@ -83,18 +85,20 @@ static void internet_Init(void)
 #endif
     initialized = -1;
     if(!res) return;
+    if(!ptr->download)
+	error(_("internet routines cannot be accessed in module"));
     initialized = 1;    
     return;
 }
 
 
-SEXP do_download(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_download(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     if(!initialized) internet_Init();
     if(initialized > 0)
 	return (*ptr->download)(call, op, args, env);
     else {
-	error("internet routines cannot be loaded");
+	error(_("internet routines cannot be loaded"));
 	return R_NilValue;
     }
 }
@@ -105,7 +109,7 @@ Rconnection R_newurl(char *description, char *mode)
     if(initialized > 0)
 	return (*ptr->newurl)(description, mode);
     else {
-	error("internet routines cannot be loaded");
+	error(_("internet routines cannot be loaded"));
 	return (Rconnection)0;
     }    
 }
@@ -116,7 +120,7 @@ Rconnection R_newsock(char *host, int port, int server, char *mode)
     if(initialized > 0)
 	return (*ptr->newsock)(host, port, server, mode);
     else {
-	error("internet routines cannot be loaded");
+	error(_("internet routines cannot be loaded"));
 	return (Rconnection)0;
     }    
 }
@@ -127,7 +131,7 @@ void *R_HTTPOpen(const char *url)
     if(initialized > 0)
 	return (*ptr->HTTPOpen)(url, 0);
     else {
-	error("internet routines cannot be loaded");
+	error(_("internet routines cannot be loaded"));
 	return NULL;
     }
 }
@@ -138,7 +142,7 @@ int   R_HTTPRead(void *ctx, char *dest, int len)
     if(initialized > 0)
 	return (*ptr->HTTPRead)(ctx, dest, len);
     else {
-	error("internet routines cannot be loaded");
+	error(_("internet routines cannot be loaded"));
 	return 0;
     }    
 }
@@ -149,7 +153,7 @@ void  R_HTTPClose(void *ctx)
     if(initialized > 0)
 	(*ptr->HTTPClose)(ctx);
     else
-	error("internet routines cannot be loaded");
+	error(_("internet routines cannot be loaded"));
 }
 
 void *R_FTPOpen(const char *url)
@@ -158,7 +162,7 @@ void *R_FTPOpen(const char *url)
     if(initialized > 0)
 	return (*ptr->FTPOpen)(url);
     else {
-	error("internet routines cannot be loaded");
+	error(_("internet routines cannot be loaded"));
 	return NULL;
     }
 }
@@ -169,7 +173,7 @@ int   R_FTPRead(void *ctx, char *dest, int len)
     if(initialized > 0)
 	return (*ptr->FTPRead)(ctx, dest, len);
     else {
-	error("internet routines cannot be loaded");
+	error(_("internet routines cannot be loaded"));
 	return 0;
     }    
 }
@@ -180,7 +184,7 @@ void  R_FTPClose(void *ctx)
     if(initialized > 0)
 	(*ptr->FTPClose)(ctx);
     else
-	error("internet routines cannot be loaded");
+	error(_("internet routines cannot be loaded"));
 }
 
 void Rsockopen(int *port)
@@ -189,7 +193,7 @@ void Rsockopen(int *port)
     if(initialized > 0)
 	(*ptr->sockopen)(port);
     else
-	error("socket routines cannot be loaded");
+	error(_("socket routines cannot be loaded"));
 }
 
 void Rsocklisten(int *sockp, char **buf, int *len)
@@ -198,7 +202,7 @@ void Rsocklisten(int *sockp, char **buf, int *len)
     if(initialized > 0)
 	(*ptr->socklisten)(sockp, buf, len);
     else
-	error("socket routines cannot be loaded");
+	error(_("socket routines cannot be loaded"));
 }
 
 void Rsockconnect(int *port, char **host)
@@ -207,7 +211,7 @@ void Rsockconnect(int *port, char **host)
     if(initialized > 0)
 	(*ptr->sockconnect)(port, host);
     else
-	error("socket routines cannot be loaded");
+	error(_("socket routines cannot be loaded"));
 }
 
 void Rsockclose(int *sockp)
@@ -216,7 +220,7 @@ void Rsockclose(int *sockp)
     if(initialized > 0)
 	(*ptr->sockclose)(sockp);
     else
-	error("socket routines cannot be loaded");
+	error(_("socket routines cannot be loaded"));
 }
 
 void Rsockread(int *sockp, char **buf, int *maxlen)
@@ -225,7 +229,7 @@ void Rsockread(int *sockp, char **buf, int *maxlen)
     if(initialized > 0)
 	(*ptr->sockread)(sockp, buf, maxlen);
     else
-	error("socket routines cannot be loaded");
+	error(_("socket routines cannot be loaded"));
 }
 
 void Rsockwrite(int *sockp, char **buf, int *start, int *end, int *len)
@@ -234,7 +238,7 @@ void Rsockwrite(int *sockp, char **buf, int *start, int *end, int *len)
     if(initialized > 0)
 	(*ptr->sockwrite)(sockp, buf, start, end, len);
     else
-	error("socket routines cannot be loaded");
+	error(_("socket routines cannot be loaded"));
 }
 
 int Rsockselect(int nsock, int *insockfd, int *ready, int *write,
@@ -244,7 +248,7 @@ int Rsockselect(int nsock, int *insockfd, int *ready, int *write,
     if(initialized > 0)
 	return (*ptr->sockselect)(nsock, insockfd, ready, write, timeout);
     else {
-	error("socket routines cannot be loaded");
+	error(_("socket routines cannot be loaded"));
 	return 0;
     }
 }
