@@ -1,8 +1,8 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2001   The R Development Core Team
- *  Copyright (C) 2002--2005  The R Foundation
+ *  Copyright (C) 1998-2006   The R Development Core Team
+ *  Copyright (C) 2002-2005  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  *
  *  A copy of the GNU General Public License is available via WWW at
  *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
- *  writing to the Free Software Foundation, Inc., 59 Temple Place,
- *  Suite 330, Boston, MA  02111-1307  USA.
+ *  writing to the Free Software Foundation, Inc., 51 Franklin Street
+ *  Fifth Floor, Boston, MA 02110-1301  USA.
  */
 
 /* <UTF8> char here is either ASCII or handled as a whole */
@@ -29,7 +29,7 @@
 #define __MAIN__
 #include "Defn.h"
 #include "Graphics.h"
-#include "Rdevices.h"		/* for InitGraphics */
+#include <Rdevices.h>		/* for InitGraphics */
 #include "IOStuff.h"
 #include "Fileio.h"
 #include "Parse.h"
@@ -69,15 +69,11 @@ void nl_Rdummy()
 void Rf_callToplevelHandlers(SEXP expr, SEXP value, Rboolean succeeded,
 			     Rboolean visible);
 
-int Rf_ParseBrowser(SEXP, SEXP);
+static int ParseBrowser(SEXP, SEXP);
 
  
-static void onpipe(int);
-
 extern void InitDynload();
 
-AccuracyInfo R_AccuracyInfo; /* This is declared here and declared extern in Defn.h */
-
 	/* Read-Eval-Print Loop [ =: REPL = repl ] with input from a file */
 
 static void R_ReplFile(FILE *fp, SEXP rho, int savestack, int browselevel)
@@ -105,7 +101,7 @@ static void R_ReplFile(FILE *fp, SEXP rho, int savestack, int browselevel)
 		PrintWarnings();
 	    break;
 	case PARSE_ERROR:
-	    error(_("syntax error: evaluating expression %d"), count);
+	    parseError(R_NilValue, count);
 	    break;
 	case PARSE_EOF:
 	    return;
@@ -135,11 +131,11 @@ char *R_PromptString(int browselevel, int type)
 		return BrowsePrompt;
 	    }
 	    return (char*)CHAR(STRING_ELT(GetOption(install("prompt"),
-						    R_NilValue), 0));
+						    R_BaseEnv), 0));
 	}
 	else {
 	    return (char*)CHAR(STRING_ELT(GetOption(install("continue"),
-						    R_NilValue), 0));
+						    R_BaseEnv), 0));
 	}
     }
 }
@@ -244,9 +240,9 @@ Rf_ReplIteration(SEXP rho, int savestack, int browselevel, R_ReplState *state)
 	    R_CurrentExpr = R_Parse1Buffer(&R_ConsoleIob, 1, &state->status);
 	    if (browselevel) {
 		    browsevalue = ParseBrowser(R_CurrentExpr, rho);
-		    if(browsevalue == 1 )
+		    if(browsevalue == 1)
 			    return(-1);
-		    if(browsevalue == 2 ) {
+		    if(browsevalue == 2) {
 			    R_IoBufferWriteReset(&R_ConsoleIob);
 			    return(0);
 		    }
@@ -272,7 +268,7 @@ Rf_ReplIteration(SEXP rho, int savestack, int browselevel, R_ReplState *state)
     case PARSE_ERROR:
 
 	    state->prompt_type = 1;
-	    error(_("syntax error"));
+	    parseError(R_NilValue, 0);
 	    R_IoBufferWriteReset(&R_ConsoleIob);
 	    return(1);
 
@@ -366,7 +362,7 @@ int R_ReplDLLdo1()
 	prompt_type = 1;
 	break;
     case PARSE_ERROR:
-	error(_("syntax error"));
+	parseError(R_NilValue, 0);
 	R_IoBufferWriteReset(&R_ConsoleIob);
 	prompt_type = 1;
 	break;
@@ -391,27 +387,265 @@ FILE* R_OpenSysInitFile(void);
 FILE* R_OpenSiteFile(void);
 FILE* R_OpenInitFile(void);
 
-static void handleInterrupt(int dummy)
+static RETSIGTYPE handleInterrupt(int dummy)
 {
     R_interrupts_pending = 1;
     signal(SIGINT, handleInterrupt);
 }
 
+
+#ifdef Win32
+static int num_caught = 0;
+
+static void win32_segv(int signum)
+{
+    /* NB: stack overflow is not an access violation on Win32 */
+    {   /* A simple customized print of the traceback */
+	SEXP trace, p, q;
+	int line = 1, i;
+	PROTECT(trace = R_GetTraceback(0));
+	if(trace != R_NilValue) {
+	    REprintf("\nTraceback:\n");
+	    for(p = trace; p != R_NilValue; p = CDR(p), line++) {
+		q = CAR(p); /* a character vector */
+		REprintf("%2d: ", line);
+		for(i = 0; i < LENGTH(q); i++)
+		    REprintf("%s", CHAR(STRING_ELT(q, i)));
+		REprintf("\n");
+	    }
+	    UNPROTECT(1);
+	}
+    }
+    num_caught++;
+    if(num_caught < 10) signal(signum, win32_segv);
+    if(signum == SIGILL)  
+	error("caught access violation - continue with care");
+    else
+	error("caught access violation - continue with care");
+}
+#endif
+
+#if defined(HAVE_SIGALTSTACK) && defined(HAVE_SIGACTION) && defined(HAVE_SIGEMPTYSET)
+
+/* NB: this really isn't safe, but suffices for experimentation for now.
+   In due course just set a flag and do this after the return.  OTOH,
+   if we do want to bail out with a core dump, need to do that here.
+
+   2005-12-17 BDR */
+
+#define CONSOLE_BUFFER_SIZE 1024
+static unsigned char  ConsoleBuf[CONSOLE_BUFFER_SIZE];
+
+static void sigactionSegv(int signum, siginfo_t *ip, void *context)
+{
+    char *s, buf[1024];
+
+    /* First check for stack overflow if we know the stack position.
+       We assume anything within 16Mb beyond the stack end is a stack overflow.
+     */
+    if(signum == SIGSEGV && (ip != (siginfo_t *)0) && 
+       (long) R_CStackStart != -1) {
+	long addr = (long) ip->si_addr;
+	long diff = (R_CStackDir > 0) ? R_CStackStart - addr:
+	    addr - R_CStackStart;
+	long upper = 0x1000000;  /* 16Mb */
+	if((long) R_CStackLimit != -1) upper += R_CStackLimit;
+	if(diff > 0 && diff < upper) {
+	    REprintf(_("Error: segfault from C stack overflow\n"));
+	    jump_to_toplevel();	
+	}    
+    }
+
+    /* need to take off stack checking as stack base has changed */
+    R_CStackLimit = (unsigned long)-1;
+
+    /* Do not translate these messages */
+    REprintf("\n *** caught %s ***\n", 
+	     signum == SIGILL ? "illegal operation" : 
+	     signum == SIGBUS ? "bus error" : "segfault");
+    if(ip != (siginfo_t *)0) {
+	if(signum == SIGILL) {
+	    
+	    switch(ip->si_code) {
+#ifdef ILL_ILLOPC
+	    case ILL_ILLOPC:
+		s = "illegal opcode";
+		break;
+#endif
+#ifdef ILL_ILLOPN
+	    case ILL_ILLOPN:
+		s = "illegal operand";
+		break;
+#endif
+#ifdef ILL_ILLADR
+	    case ILL_ILLADR:
+		s = "illegal addressing mode";
+		break;
+#endif
+#ifdef ILL_ILLTRP
+	    case ILL_ILLTRP:
+		s = "illegal trap";
+		break;
+#endif
+#ifdef ILL_COPROC
+	    case ILL_COPROC:
+		s = "coprocessor error";
+		break;
+#endif
+	    default:
+		s = "unknown";
+		break;
+	    }
+	} else if(signum == SIGBUS)
+	    switch(ip->si_code) {
+#ifdef BUS_ADRALN
+	    case BUS_ADRALN:
+		s = "invalid alignment";
+		break;
+#endif
+#ifdef BUS_ADRERR /* not on MacOS X, apparently */
+	    case BUS_ADRERR:
+		s = "non-existent physical address";
+		break;
+#endif
+#ifdef BUS_OBJERR /* not on MacOS X, apparently */
+	    case BUS_OBJERR:
+		s = "object specific hardware error";
+		break;
+#endif
+	    default:
+		s = "unknown";
+		break;
+	    }
+	else
+	    switch(ip->si_code) {
+#ifdef SEGV_MAPERR
+	    case SEGV_MAPERR:
+		s = "memory not mapped";
+		break;
+#endif
+#ifdef SEGV_ACCERR
+	    case SEGV_ACCERR:
+		s = "invalid permissions";
+		break;
+#endif
+	    default:
+		s = "unknown";
+		break;
+	    }
+	REprintf("address %p, cause '%s'\n", ip->si_addr, s);
+    }
+    {   /* A simple customized print of the traceback */
+	SEXP trace, p, q;
+	int line = 1, i;
+	PROTECT(trace = R_GetTraceback(0));
+	if(trace != R_NilValue) {
+	    REprintf("\nTraceback:\n");
+	    for(p = trace; p != R_NilValue; p = CDR(p), line++) {
+		q = CAR(p); /* a character vector */
+		REprintf("%2d: ", line);
+		for(i = 0; i < LENGTH(q); i++)
+		    REprintf("%s", CHAR(STRING_ELT(q, i)));
+		REprintf("\n");
+	    }
+	    UNPROTECT(1);
+	}
+    }
+    if(R_Interactive) {
+	REprintf("\nPossible actions:\n1: %s\n2: %s\n3: %s\n4: %s\n", 
+		 "abort (with core dump)", 
+		 "normal R exit", 
+		 "exit R without saving workspace",
+		 "exit R saving workspace");
+	while(1) {
+	    if(R_ReadConsole("Selection: ", ConsoleBuf, CONSOLE_BUFFER_SIZE, 
+			     0) > 0) {
+		if(ConsoleBuf[0] == '1') break;
+		if(ConsoleBuf[0] == '2') R_CleanUp(SA_DEFAULT, 0, 1);
+		if(ConsoleBuf[0] == '3') R_CleanUp(SA_NOSAVE, 70, 0);
+		if(ConsoleBuf[0] == '4') R_CleanUp(SA_SAVE, 71, 0);
+	    }
+	}
+    }
+    REprintf("aborting ...\n");
+    snprintf(buf, 1024, "rm -rf %s", R_TempDir);
+    R_system(buf);
+    /* now do normal behaviour, e.g. core dump */
+    signal(signum, SIG_DFL);
+    raise(signum);
+}
+
+#ifndef SIGSTKSZ
+# define SIGSTKSZ 8192    /* just a guess */
+#endif
+
+#ifdef HAVE_STACK_T
+static stack_t sigstk;
+#else
+static struct sigaltstack sigstk;
+#endif
+static void *signal_stack;
+
+#define R_USAGE 100000 /* Just a guess */
+static void init_signal_handlers()
+{
+    /* <FIXME> may need to reinstall this if we do recover. */
+    struct sigaction sa;
+    signal_stack = malloc(SIGSTKSZ + R_USAGE);
+    if (signal_stack != NULL) {
+        sigstk.ss_sp = signal_stack;
+        sigstk.ss_size = SIGSTKSZ + R_USAGE;
+        sigstk.ss_flags = 0;
+        if(sigaltstack(&sigstk, NULL) < 0)
+	    warning("failed to set alternate signal stack");
+    } else
+	warning("failed to allocate alternate signal stack");
+    sa.sa_sigaction = sigactionSegv;
+    sigemptyset(&sa.sa_mask);
+    sa.sa_flags = SA_ONSTACK | SA_SIGINFO;
+    sigaction(SIGSEGV, &sa, NULL);
+    sigaction(SIGILL, &sa, NULL);
+#ifdef SIGBUS
+    sigaction(SIGBUS, &sa, NULL);
+#endif
+
+    signal(SIGINT,  handleInterrupt);
+    signal(SIGUSR1, onsigusr1);
+    signal(SIGUSR2, onsigusr2);
+    signal(SIGPIPE, SIG_IGN);
+}
+
+#else /* not sigaltstack and sigaction and sigemptyset*/
+static void init_signal_handlers()
+{
+    signal(SIGINT,  handleInterrupt);
+    signal(SIGUSR1, onsigusr1);
+    signal(SIGUSR2, onsigusr2);
+#ifndef Win32
+    signal(SIGPIPE, SIG_IGN);
+#else
+    signal(SIGSEGV, win32_segv);
+    signal(SIGILL, win32_segv);
+#endif
+}
+#endif
+
+
 static void R_LoadProfile(FILE *fparg, SEXP env)
 {
     FILE * volatile fp = fparg; /* is this needed? */
     if (fp != NULL) {
 	if (! SETJMP(R_Toplevel.cjmpbuf)) {
 	    R_GlobalContext = R_ToplevelContext = &R_Toplevel;
-#ifdef REINSTALL_SIGNAL_HANDLERS
-	    signal(SIGINT, handleInterrupt);
-#endif
 	    R_ReplFile(fp, env, 0, 0);
 	}
 	fclose(fp);
     }
 }
 
+
+int R_SignalHandlers = 1;  /* Exposed in R_interface.h */
+
 /* Use this to allow e.g. Win32 malloc to call warning.
    Don't use R-specific type, e.g. Rboolean */
 /* int R_Is_Running = 0; now in Defn.h */
@@ -438,6 +672,9 @@ void setup_Rmainloop(void)
 	if(p) strcpy(Rlocale, p); else strcpy(Rlocale, "");
 	if((p = getenv("LC_CTYPE"))) setlocale(LC_CTYPE, p);
 	else setlocale(LC_CTYPE, Rlocale);
+	/* LC_CTYPE=C bombs in mingwex */
+	if(strcmp(setlocale(LC_CTYPE, NULL), "C") == 0) 
+	    setlocale(LC_CTYPE, "en");
 	if((p = getenv("LC_COLLATE"))) setlocale(LC_COLLATE, p);
 	else setlocale(LC_COLLATE, Rlocale);
 	if((p = getenv("LC_TIME"))) setlocale(LC_TIME, p);
@@ -459,18 +696,26 @@ void setup_Rmainloop(void)
 #ifdef ENABLE_NLS
     /* This ought to have been done earlier, but be sure */
     textdomain(PACKAGE);
-    strcpy(localedir, R_Home); strcat(localedir, "/share/locale");
+    {
+	char *p = getenv("R_SHARE_DIR");
+	if(p) {
+	    strcpy(localedir, p);
+	    strcat(localedir, "/locale");
+	} else {
+	    strcpy(localedir, R_Home);
+	    strcat(localedir, "/share/locale");
+	}
+    }
     bindtextdomain(PACKAGE, localedir);
     strcpy(localedir, R_Home); strcat(localedir, "/library/base/po");
     bindtextdomain("R-base", localedir);
 #endif
 #endif
 
-#if defined(Unix) || defined(Win32)
     InitTempDir(); /* must be before InitEd */
-#endif
     InitMemory();
     InitNames();
+    InitBaseEnv();
     InitGlobalEnv();
     InitDynload();
     InitOptions();
@@ -485,6 +730,13 @@ void setup_Rmainloop(void)
 #ifdef SUPPORT_MBCS
     mbcslocale = MB_CUR_MAX > 1;
 #endif
+#ifdef Win32
+    {
+	char *ctype = setlocale(LC_CTYPE, NULL), *p;
+	p = strrchr(ctype, '.');
+	if(p && isdigit(p[1])) localeCP = atoi(p+1); else localeCP = 1252;
+    }
+#endif
 #if defined(Win32) && defined(SUPPORT_UTF8)
     utf8locale = mbcslocale = TRUE;
 #endif
@@ -500,8 +752,8 @@ void setup_Rmainloop(void)
     R_Toplevel.promargs = R_NilValue;
     R_Toplevel.callfun = R_NilValue;
     R_Toplevel.call = R_NilValue;
-    R_Toplevel.cloenv = R_NilValue;
-    R_Toplevel.sysparent = R_NilValue;
+    R_Toplevel.cloenv = R_BaseEnv;
+    R_Toplevel.sysparent = R_BaseEnv;
     R_Toplevel.conexit = R_NilValue;
     R_Toplevel.vmax = NULL;
 #ifdef BYTECODE
@@ -532,19 +784,13 @@ void setup_Rmainloop(void)
     */
 
     fp = R_OpenLibraryFile("base");
-    if (fp == NULL) {
+    if (fp == NULL)
 	R_Suicide(_("unable to open the base package\n"));
-    }
 
     doneit = 0;
     SETJMP(R_Toplevel.cjmpbuf);
     R_GlobalContext = R_ToplevelContext = &R_Toplevel;
-    signal(SIGINT, handleInterrupt);
-    signal(SIGUSR1,onsigusr1);
-    signal(SIGUSR2,onsigusr2);
-#ifdef Unix
-    signal(SIGPIPE, onpipe);
-#endif
+    if (R_SignalHandlers) init_signal_handlers();
     if (!doneit) {
 	doneit = 1;
 	R_ReplFile(fp, baseEnv, 0, 0);
@@ -588,14 +834,6 @@ void setup_Rmainloop(void)
     doneit = 0;
     SETJMP(R_Toplevel.cjmpbuf);
     R_GlobalContext = R_ToplevelContext = &R_Toplevel;
-#ifdef REINSTALL_SIGNAL_HANDLERS
-    signal(SIGINT, handleInterrupt);
-    signal(SIGUSR1,onsigusr1);
-    signal(SIGUSR2,onsigusr2);
-#ifdef Unix
-    signal(SIGPIPE, onpipe);
-#endif
-#endif
     if (!doneit) {
 	doneit = 1;
 	R_InitialData();
@@ -610,9 +848,6 @@ void setup_Rmainloop(void)
     doneit = 0;
     SETJMP(R_Toplevel.cjmpbuf);
     R_GlobalContext = R_ToplevelContext = &R_Toplevel;
-#ifdef REINSTALL_SIGNAL_HANDLERS
-    signal(SIGINT, handleInterrupt);
-#endif
     if (!doneit) {
 	doneit = 1;
 	PROTECT(cmd = install(".First"));
@@ -631,9 +866,6 @@ void setup_Rmainloop(void)
     doneit = 0;
     SETJMP(R_Toplevel.cjmpbuf);
     R_GlobalContext = R_ToplevelContext = &R_Toplevel;
-#ifdef REINSTALL_SIGNAL_HANDLERS
-    signal(SIGINT, handleInterrupt);
-#endif
     if (!doneit) {
 	doneit = 1;
 	PROTECT(cmd = install(".First.sys"));
@@ -653,20 +885,20 @@ void setup_Rmainloop(void)
     }
 }
 
+extern SA_TYPE SaveAction; /* from src/main/startup.c */
+
 void end_Rmainloop(void)
 {
-    Rprintf("\n");
+    /* refrain from printing trailing '\n' only if quiet flag is on and
+       the save action is known (e.g. implied by --slave) */
+    if (!R_Quiet ||
+	(SaveAction != SA_NOSAVE && SaveAction != SA_SAVE))
+        Rprintf("\n");
     /* run the .Last function. If it gives an error, will drop back to main
        loop. */
     R_CleanUp(SA_DEFAULT, 0, 1);
 }
 
-static void onpipe(int dummy)
-{
-    /* do nothing */
-    signal(SIGPIPE, onpipe);
-}
-
 void run_Rmainloop(void)
 {
     /* Here is the real R read-eval-loop. */
@@ -674,14 +906,6 @@ void run_Rmainloop(void)
     R_IoBufferInit(&R_ConsoleIob);
     SETJMP(R_Toplevel.cjmpbuf);
     R_GlobalContext = R_ToplevelContext = &R_Toplevel;
-#ifdef REINSTALL_SIGNAL_HANDLERS
-    signal(SIGINT, handleInterrupt);
-    signal(SIGUSR1,onsigusr1);
-    signal(SIGUSR2,onsigusr2);
-#ifdef Unix
-    signal(SIGPIPE, onpipe);
-#endif
-#endif
     R_ReplConsole(R_GlobalEnv, 0, 0);
     end_Rmainloop(); /* must go here */
 }
@@ -701,9 +925,9 @@ static void printwhere(void)
   int lct = 1;
 
   for (cptr = R_GlobalContext; cptr; cptr = cptr->nextcontext) {
-    if ((cptr->callflag & CTXT_FUNCTION) &&
+    if ((cptr->callflag & (CTXT_FUNCTION | CTXT_BUILTIN)) &&
 	(TYPEOF(cptr->call) == LANGSXP)) {
-	Rprintf("where %d: ",lct++);
+	Rprintf("where %d: ", lct++);
 	PrintValue(cptr->call);
     }
   }
@@ -711,23 +935,23 @@ static void printwhere(void)
 }
 
 
-int Rf_ParseBrowser(SEXP CExpr, SEXP rho)
+static int ParseBrowser(SEXP CExpr, SEXP rho)
 {
-    int rval=0;
+    int rval = 0;
     if (isSymbol(CExpr)) {
-	if (!strcmp(CHAR(PRINTNAME(CExpr)),"n")) {
+	if (!strcmp(CHAR(PRINTNAME(CExpr)), "n")) {
 	    SET_DEBUG(rho, 1);
-	    rval=1;
+	    rval = 1;
 	}
-	if (!strcmp(CHAR(PRINTNAME(CExpr)),"c")) {
-	    rval=1;
+	if (!strcmp(CHAR(PRINTNAME(CExpr)), "c")) {
+	    rval = 1;
 	    SET_DEBUG(rho, 0);
 	}
-	if (!strcmp(CHAR(PRINTNAME(CExpr)),"cont")) {
-	    rval=1;
+	if (!strcmp(CHAR(PRINTNAME(CExpr)), "cont")) {
+	    rval = 1;
 	    SET_DEBUG(rho, 0);
 	}
-	if (!strcmp(CHAR(PRINTNAME(CExpr)),"Q")) {
+	if (!strcmp(CHAR(PRINTNAME(CExpr)), "Q")) {
 
 	    /* Run onexit/cend code for everything above the target.
                The browser context is still on the stack, so any error
@@ -739,14 +963,14 @@ int Rf_ParseBrowser(SEXP CExpr, SEXP rho)
 
 	    /* this is really dynamic state that should be managed as such */
 	    R_BrowseLevel = 0;
-	    SET_DEBUG(rho,0); /*PR#1721*/
+	    SET_DEBUG(rho, 0); /*PR#1721*/
 
 	    jump_to_toplevel();
 	}
 	if (!strcmp(CHAR(PRINTNAME(CExpr)),"where")) {
 	    printwhere();
-	    SET_DEBUG(rho, 1);
-	    rval=2;
+	    /* SET_DEBUG(rho, 1); */
+	    rval = 2;
 	}
     }
     return rval;
@@ -760,13 +984,12 @@ static void browser_cend(void *data)
     R_BrowseLevel = *psaved - 1;
 }
 
-SEXP do_browser(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_browser(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     RCNTXT *saveToplevelContext;
     RCNTXT *saveGlobalContext;
     RCNTXT thiscontext, returncontext, *cptr;
-    int savestack;
-    int savebrowselevel;
+    int savestack, savebrowselevel, tmp;
     SEXP topExp;
 
     /* Save the evaluator state information */
@@ -779,11 +1002,14 @@ SEXP do_browser(SEXP call, SEXP op, SEXP args, SEXP rho)
     saveGlobalContext = R_GlobalContext;
 
     if (!DEBUG(rho)) {
-	cptr=R_GlobalContext;
+	cptr = R_GlobalContext;
 	while ( !(cptr->callflag & CTXT_FUNCTION) && cptr->callflag )
 	    cptr = cptr->nextcontext;
 	Rprintf("Called from: ");
+	tmp = asInteger(GetOption(install("deparse.max.lines"), R_BaseEnv));
+	if(tmp != NA_INTEGER && tmp > 0) R_BrowseLines = tmp;
 	PrintValueRec(cptr->call,rho);
+	R_BrowseLines = 0;
     }
 
     R_ReturnedValue = R_NilValue;
@@ -795,12 +1021,12 @@ SEXP do_browser(SEXP call, SEXP op, SEXP args, SEXP rho)
     /* acts as a target for error returns. */
 
     begincontext(&returncontext, CTXT_BROWSER, call, rho,
-		 R_NilValue, R_NilValue, R_NilValue);
+		 R_BaseEnv, R_NilValue, R_NilValue);
     returncontext.cend = browser_cend;
     returncontext.cenddata = &savebrowselevel;
     if (!SETJMP(returncontext.cjmpbuf)) {
 	begincontext(&thiscontext, CTXT_RESTART, R_NilValue, rho,
-		     R_NilValue, R_NilValue, R_NilValue);
+		     R_BaseEnv, R_NilValue, R_NilValue);
 	if (SETJMP(thiscontext.cjmpbuf)) {
 	    SET_RESTART_BIT_ON(thiscontext.callflag);
 	    R_ReturnedValue = R_NilValue;
@@ -811,9 +1037,6 @@ SEXP do_browser(SEXP call, SEXP op, SEXP args, SEXP rho)
 	R_InsertRestartHandlers(&thiscontext, TRUE);
 #endif
 	R_BrowseLevel = savebrowselevel;
-#ifdef REINSTALL_SIGNAL_HANDLERS
-	signal(SIGINT, handleInterrupt);
-#endif
 	R_ReplConsole(rho, savestack, R_BrowseLevel);
 	endcontext(&thiscontext);
     }
@@ -849,7 +1072,7 @@ void R_dot_Last(void)
     UNPROTECT(1);
 }
 
-SEXP do_quit(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_quit(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     char *tmp;
     int ask=SA_DEFAULT, status, runLast;
