@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 /* <UTF8> the only interpretation of char is ASCII */
@@ -45,6 +45,8 @@ static void  in_R_FTPClose(void *ctx);
 
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
+/* Solaris and AIX define open as open64 under some circumstances */
+# undef open
 #endif
 
 /* ------------------- internet access functions  --------------------- */
@@ -219,6 +221,21 @@ typedef struct {
 
 #ifdef Win32
 #include <graphapp/ga.h>
+
+typedef struct {
+    window wprog;
+    progressbar pb;
+    label l_url;
+    RCNTXT cntxt;
+} winprogressbar;
+
+static winprogressbar pbar = {NULL, NULL, NULL};
+
+static void doneprogressbar(void *data)
+{
+    winprogressbar *pbar = data;
+    hide(pbar->wprog);
+}
 #endif
 
 /* download(url, destfile, quiet, mode, cacheOK) */
@@ -230,44 +247,52 @@ static SEXP in_do_download(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP ans, scmd, sfile, smode;
     char *url, *file, *mode;
     int quiet, status = 0, cacheOK;
-#ifdef Win32
-    window wprog;
-    progressbar pb;
-    label l_url;
-#endif
-
 
     checkArity(op, args);
     scmd = CAR(args); args = CDR(args);
     if(!isString(scmd) || length(scmd) < 1)
-	error(_("invalid 'url' argument"));
+	error(_("invalid '%s' argument"), "url");
     if(length(scmd) > 1)
 	warning(_("only first element of 'url' argument used"));
     url = CHAR(STRING_ELT(scmd, 0));
     sfile = CAR(args); args = CDR(args);
     if(!isString(sfile) || length(sfile) < 1)
-	error(_("invalid 'destfile' argument"));
+	error(_("invalid '%s' argument"), "destfile");
     if(length(sfile) > 1)
 	warning(_("only first element of 'destfile' argument used"));
     file = CHAR(STRING_ELT(sfile, 0));
     IDquiet = quiet = asLogical(CAR(args)); args = CDR(args);
     if(quiet == NA_LOGICAL)
-	error(_("invalid 'quiet' argument"));
+	error(_("invalid '%s' argument"), "quiet");
     smode =  CAR(args); args = CDR(args);
     if(!isString(smode) || length(smode) != 1)
-	error(_("invalid 'mode' argument"));
+	error(_("invalid '%s' argument"), "mode");
     mode = CHAR(STRING_ELT(smode, 0));
     cacheOK = asLogical(CAR(args));
     if(cacheOK == NA_LOGICAL)
-	error(_("invalid 'cacheOK' argument"));
-
+	error(_("invalid '%s' argument"), "cacheOK");
+#ifdef Win32
+    if (!pbar.wprog) {
+	pbar.wprog = newwindow(_("Download progress"), rect(0, 0, 540, 100),
+		      Titlebar | Centered);
+	setbackground(pbar.wprog, dialog_bg());
+	pbar.l_url = newlabel(" ", rect(10, 15, 520, 25), AlignCenter);
+	pbar.pb = newprogressbar(rect(20, 50, 500, 20), 0, 1024, 1024, 1);	    	
+    }
+#endif
     if(strncmp(url, "file://", 7) == 0) {
 	FILE *in, *out;
 	static char buf[CPBUFSIZE];
 	size_t n;
+	int nh = 7;
+#ifdef Win32
+	/* on Windows we have file:///d:/path/to 
+	   whereas on Unix it is file:///path/to */
+	if (strlen(url) > 9 && url[7] == '/' && url[9] == ':') nh = 8;
+#endif
 
 	/* Use binary transfers */
-	in = R_fopen(R_ExpandFileName(url+7), (mode[2] == 'b') ? "rb" : "r");
+	in = R_fopen(R_ExpandFileName(url+nh), (mode[2] == 'b') ? "rb" : "r");
 	if(!in) error(_("cannot open URL '%s'"), url);
 	out = R_fopen(R_ExpandFileName(file), mode);
 	if(!out) error(_("cannot open destfile '%s'"), file);
@@ -302,17 +327,18 @@ static SEXP in_do_download(SEXP call, SEXP op, SEXP args, SEXP env)
 #ifdef Win32
 	    if (guess <= 0) guess = 100 * 1024;
 	    R_FlushConsole();
-	    wprog = newwindow(_("Download progress"), rect(0, 0, 540, 100),
-			      Titlebar | Centered);
-	    setbackground(wprog, dialog_bg());
 	    strcpy(buf, "URL: ");
 	    if(strlen(url) > 60) {
 		strcat(buf, "... ");
 		strcat(buf, url + (strlen(url) - 60));
 	    } else strcat(buf, url);
-	    l_url = newlabel(buf, rect(10, 15, 520, 25), AlignCenter);
-	    pb = newprogressbar(rect(20, 50, 500, 20), 0, guess, 1024, 1);
-	    show(wprog);
+	    settext(pbar.l_url, buf);
+	    setprogressbarrange(pbar.pb, 0, guess);
+	    show(pbar.wprog);
+	    begincontext(&(pbar.cntxt), CTXT_CCODE, R_NilValue, R_NilValue,
+			 R_NilValue, R_NilValue, R_NilValue);
+	    pbar.cntxt.cend = &doneprogressbar;
+	    pbar.cntxt.cenddata = &pbar;
 #endif
 	    while ((len = in_R_HTTPRead(ctxt, buf, sizeof(buf))) > 0) {
 		fwrite(buf, 1, len, out);
@@ -320,9 +346,9 @@ static SEXP in_do_download(SEXP call, SEXP op, SEXP args, SEXP env)
 #ifdef Win32
 		if(nbytes > guess) {
 		    guess *= 2;
-		    setprogressbarrange(pb, 0, guess);
+		    setprogressbarrange(pbar.pb, 0, guess);
 		}
-		setprogressbar(pb, nbytes);
+		setprogressbar(pbar.pb, nbytes);
 #else
 		if(!quiet) {
 		    if(guess <= 0) putdots(&ndots, nbytes/1024);
@@ -343,10 +369,8 @@ static SEXP in_do_download(SEXP call, SEXP op, SEXP args, SEXP env)
 	    }
 #ifdef Win32
 	    R_FlushConsole();
-	    hide(wprog);
-	    del(l_url);
-	    del(pb);
-	    del(wprog);
+	    endcontext(&(pbar.cntxt));
+	    doneprogressbar(&pbar);	    
 #endif
 	    if (total > 0 && total != nbytes)
 		warning(_("downloaded length %d != reported length %d"),
@@ -381,17 +405,21 @@ static SEXP in_do_download(SEXP call, SEXP op, SEXP args, SEXP env)
 #ifdef Win32
 	    if (guess <= 0) guess = 100 * 1024;
 	    R_FlushConsole();
-	    wprog = newwindow(_("Download progress"), rect(0, 0, 540, 100),
-			      Titlebar | Centered);
-	    setbackground(wprog, LightGrey);
 	    strcpy(buf, "URL: ");
 	    if(strlen(url) > 60) {
 		strcat(buf, "... ");
 		strcat(buf, url + (strlen(url) - 60));
 	    } else strcat(buf, url);
-	    l_url = newlabel(buf, rect(10, 15, 520, 25), AlignCenter);
-	    pb = newprogressbar(rect(20, 50, 500, 20), 0, guess, 1024, 1);
-	    show(wprog);
+	    settext(pbar.l_url, buf);
+	    setprogressbarrange(pbar.pb, 0, guess);
+	    show(pbar.wprog);
+
+	    /* set up a context which will close progressbar on error. */
+	    begincontext(&(pbar.cntxt), CTXT_CCODE, R_NilValue, R_NilValue,
+			 R_NilValue, R_NilValue, R_NilValue);
+	    pbar.cntxt.cend = &doneprogressbar;
+	    pbar.cntxt.cenddata = &pbar;
+	    
 #endif
 	    while ((len = in_R_FTPRead(ctxt, buf, sizeof(buf))) > 0) {
 		fwrite(buf, 1, len, out);
@@ -399,9 +427,9 @@ static SEXP in_do_download(SEXP call, SEXP op, SEXP args, SEXP env)
 #ifdef Win32
 		if(nbytes > guess) {
 		    guess *= 2;
-		    setprogressbarrange(pb, 0, guess);
+		    setprogressbarrange(pbar.pb, 0, guess);
 		}
-		setprogressbar(pb, nbytes);
+		setprogressbar(pbar.pb, nbytes);
 #else
 		if(!quiet) {
 		    if(guess <= 0) putdots(&ndots, nbytes/1024);
@@ -422,10 +450,8 @@ static SEXP in_do_download(SEXP call, SEXP op, SEXP args, SEXP env)
 	    }
 #ifdef Win32
 	    R_FlushConsole();
-	    hide(wprog);
-	    del(l_url);
-	    del(pb);
-	    del(wprog);
+	    endcontext(&(pbar.cntxt));
+	    doneprogressbar(&pbar);	    
 #endif
 	    if (total > 0 && total != nbytes)
 		warning(_("downloaded length %d != reported length %d"),
@@ -451,7 +477,7 @@ void *in_R_HTTPOpen(const char *url, int cacheOK)
 {
     inetconn *con;
     void *ctxt;
-    int timeout = asInteger(GetOption(install("timeout"), R_NilValue));
+    int timeout = asInteger(GetOption(install("timeout"), R_BaseEnv));
     int len = -1;
     char *type = NULL;
 
@@ -462,9 +488,11 @@ void *in_R_HTTPOpen(const char *url, int cacheOK)
     if(ctxt != NULL) {
 	int rc = RxmlNanoHTTPReturnCode(ctxt);
 	if(rc != 200) {
+	    char *msg;
 	    RxmlNanoHTTPClose(ctxt);
-	    warning(_("cannot open: HTTP status was '%d %s'"), rc,
-		  RxmlNanoHTTPStatusMsg(ctxt));
+	    /* bug work-around: it will crash on OS X if passed directly */
+	    msg = _("cannot open: HTTP status was '%d %s'");
+	    warning(msg, rc, RxmlNanoHTTPStatusMsg(ctxt));
 	    return NULL;
 	} else {
 	    type = RxmlNanoHTTPContentType(ctxt);
@@ -505,7 +533,7 @@ static void *in_R_FTPOpen(const char *url)
 {
     inetconn *con;
     void *ctxt;
-    int timeout = asInteger(GetOption(install("timeout"), R_NilValue));
+    int timeout = asInteger(GetOption(install("timeout"), R_BaseEnv));
     int len = 0;
 
     if(timeout == NA_INTEGER || timeout <= 0) timeout = 60;
@@ -548,6 +576,7 @@ static void in_R_FTPClose(void *ctx)
 
 #ifdef USE_WININET
 
+#define WIN32_LEAN_AND_MEAN 1
 #include <windows.h>
 #include <wininet.h>
 typedef struct wictxt {
@@ -607,7 +636,7 @@ static void *in_R_HTTPOpen(const char *url, const int cacheOK)
     }
 
 #ifdef USE_WININET_ASYNC
-    timeout = asInteger(GetOption(install("timeout"), R_NilValue));
+    timeout = asInteger(GetOption(install("timeout"), R_BaseEnv));
     if(timeout == NA_INTEGER || timeout <= 0) timeout = 60;
     InternetSetStatusCallback(wictxt->hand,
 			      (INTERNET_STATUS_CALLBACK) InternetCallback);
@@ -764,7 +793,7 @@ static void *in_R_FTPOpen(const char *url)
     }
 
 #ifdef USE_WININET_ASYNC
-    timeout = asInteger(GetOption(install("timeout"), R_NilValue));
+    timeout = asInteger(GetOption(install("timeout"), R_BaseEnv));
     if(timeout == NA_INTEGER || timeout <= 0) timeout = 60;
     InternetSetStatusCallback(wictxt->hand,
 			      (INTERNET_STATUS_CALLBACK) InternetCallback);
@@ -878,7 +907,7 @@ void RxmlMessage(int level, const char *format, ...)
     char buf[MBUFSIZE], *p;
     va_list(ap);
 
-    clevel = asInteger(GetOption(install("internet.info"), R_NilValue));
+    clevel = asInteger(GetOption(install("internet.info"), R_BaseEnv));
     if(clevel == NA_INTEGER) clevel = 2;
     
     if(level < clevel) return;
@@ -893,19 +922,23 @@ void RxmlMessage(int level, const char *format, ...)
 }
 
 #include "sock.h"
+#ifdef Win32
+# define STRICT_R_HEADERS
+#endif
+#include <R_ext/RS.h> /* for Calloc */
 
+void
+#ifdef HAVE_VISIBILITY_ATTRIBUTE
+__attribute__ ((visibility ("default")))
+#endif
 #ifdef USE_WININET
-void R_init_internet2(DllInfo *info)
+R_init_internet2(DllInfo *info)
 #else
-void R_init_internet(DllInfo *info)
+R_init_internet(DllInfo *info)
 #endif
 {
     R_InternetRoutines *tmp;
-    tmp = (R_InternetRoutines*) malloc(sizeof(R_InternetRoutines));
-    if(!tmp) {
-	error(_("cannot allocate memory for InternetRoutines structure"));
-	return;
-    }
+    tmp = Calloc(1, R_InternetRoutines);
 
     tmp->download = in_do_download;
     tmp->newurl =  in_R_newurl;
