@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001--2003  The R Development Core Team.
- *  Copyright (C) 2003-4      The R Foundation
+ *  Copyright (C) 2001--2006  The R Development Core Team.
+ *  Copyright (C) 2003-5      The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,14 +15,17 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
+/* <UTF8> All chars are ASCII */
+
 /* Interface routines, callable from R using .Call, for Lapack code */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
+#include <Defn.h>
 
 #include "Lapack.h"
 
@@ -35,14 +38,14 @@ static SEXP modLa_svd(SEXP jobu, SEXP jobv, SEXP x, SEXP s, SEXP u, SEXP v,
     char *meth;
 
     if (!(isString(jobu) && isString(jobv)))
-	error("jobu and jobv must be character objects");
+	error(_("'jobu' and 'jobv' must be character strings"));
     if (!isString(method))
-	error("method must be a character object");
+	error(_("'method' must be a character string"));
     meth = CHAR(STRING_ELT(method, 0));
-#ifndef IEEE_754
+/*#ifndef IEEE_754
     if (strcmp(meth, "dgesdd") == 0)
 	error("method = \"dgesdd\" requires IEEE 754 arithmetic");
-#endif
+#endif*/
     xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
     n = xdims[0]; p = xdims[1];
     xvals = (double *) R_alloc(n * p, sizeof(double));
@@ -58,7 +61,7 @@ static SEXP modLa_svd(SEXP jobu, SEXP jobv, SEXP x, SEXP s, SEXP u, SEXP v,
 			 REAL(v), INTEGER(getAttrib(v, R_DimSymbol)),
 			 &tmp, &lwork, &info);
 	if (info != 0)
-	    error("error code %d from Lapack routine dgesvd", info);
+	    error(_("error code %d from Lapack routine '%s'"), info, "dgesvd");
 	lwork = (int) tmp;
 
 	work = (double *) R_alloc(lwork, sizeof(double));
@@ -68,7 +71,7 @@ static SEXP modLa_svd(SEXP jobu, SEXP jobv, SEXP x, SEXP s, SEXP u, SEXP v,
 			 REAL(v), INTEGER(getAttrib(v, R_DimSymbol)),
 			 work, &lwork, &info);
 	if (info != 0)
-	    error("error code %d from Lapack routine dgesvd", info);
+	    error(_("error code %d from Lapack routine '%s'"), info, "dgesvd");
     } else {
 	int ldu = INTEGER(getAttrib(u, R_DimSymbol))[0],
 	    ldvt = INTEGER(getAttrib(v, R_DimSymbol))[0];
@@ -82,7 +85,7 @@ static SEXP modLa_svd(SEXP jobu, SEXP jobv, SEXP x, SEXP s, SEXP u, SEXP v,
 			 REAL(v), &ldvt,
 			 &tmp, &lwork, iwork, &info);
 	if (info != 0)
-	    error("error code %d from Lapack routine dgesdd", info);
+	    error(_("error code %d from Lapack routine '%s'"), info, "dgesdd");
 	lwork = (int) tmp;
 	work = (double *) R_alloc(lwork, sizeof(double));
 	F77_CALL(dgesdd)(CHAR(STRING_ELT(jobu, 0)),
@@ -91,7 +94,7 @@ static SEXP modLa_svd(SEXP jobu, SEXP jobv, SEXP x, SEXP s, SEXP u, SEXP v,
 			 REAL(v), &ldvt,
 			 work, &lwork, iwork, &info);
 	if (info != 0)
-	    error("error code %d from Lapack routine dgesdd", info);
+	    error(_("error code %d from Lapack routine '%s'"), info, "dgesdd");
     }
 
     val = PROTECT(allocVector(VECSXP, 3));
@@ -107,109 +110,61 @@ static SEXP modLa_svd(SEXP jobu, SEXP jobv, SEXP x, SEXP s, SEXP u, SEXP v,
     return val;
 }
 
-static SEXP modLa_rs(SEXP xin, SEXP only_values, SEXP method)
+static SEXP modLa_rs(SEXP xin, SEXP only_values)
 {
     int *xdims, n, lwork, info = 0, ov;
     char jobv[1], uplo[1], range[1];
     SEXP values, ret, nm, x, z = R_NilValue;
     double *work, *rx, *rvalues, tmp;
-    char *meth;
+    int liwork, *iwork, itmp, m;
+    double vl = 0.0, vu = 0.0, abstol = 0.0;
+    /* valgrind seems to think vu should be set, but it is documented
+       not to be used if range='a' */
+    int il, iu, *isuppz;
 
-    if (!isString(method))
-	error("method must be a character object");
-    meth = CHAR(STRING_ELT(method, 0));
-#ifndef IEEE_754
-    if (strcmp(meth, "dsyevr") == 0) {
-	warning("method = \"dseyvr\" requires IEEE 754 arithmetic: using \"dsyev\"");
-	meth = "dsyev";
-    }
-#endif
     PROTECT(x = duplicate(xin));
     rx = REAL(x);
     uplo[0] = 'L';
     xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
     n = xdims[0];
     if (n != xdims[1])
-	error("x must be a square numeric matrix");
+	error(_("'x' must be a square numeric matrix"));
     ov = asLogical(only_values);
-    if (ov == NA_LOGICAL) error("invalid `only.values'");
+    if (ov == NA_LOGICAL) error(_("invalid 'only.values'"));
     if (ov) jobv[0] = 'N'; else jobv[0] = 'V';
 
     PROTECT(values = allocVector(REALSXP, n));
     rvalues = REAL(values);
-    if(strcmp(meth, "dsyevr")) {
-	/* ask for optimal size of work array */
-	lwork = -1;
-#ifdef HAVE_LAPACK
-	F77_CALL(dsyev)(jobv, uplo, &n, rx, &n, rvalues, &tmp, &lwork, &info);
-#else
-        F77_CALL(rsyev)(jobv, uplo, &n, rx, &n, rvalues, &tmp, &lwork, &info);
-#endif
-	if (info != 0)
-	    error("error code %d from Lapack routine dsyev", info);
-	lwork = (int) tmp;
-	if (lwork < 3*n-1) lwork = 3*n-1;  /* Sanity check */
-	work = (double *) R_alloc(lwork, sizeof(double));
-#ifdef HAVE_LAPACK
-	F77_CALL(dsyev)(jobv, uplo, &n, rx, &n, rvalues, work, &lwork, &info);
-#else
-        F77_CALL(rsyev)(jobv, uplo, &n, rx, &n, rvalues, work, &lwork, &info);
-#endif
-	if (info != 0)
-	    error("error code %d from Lapack routine dsyev", info);
-    } else {
-	int liwork, *iwork, itmp, m;
-	double vl, vu, abstol = 0.0;
-	int il, iu, *isuppz;
 
-	range[0] = 'A';
-	if (!ov) PROTECT(z = allocMatrix(REALSXP, n, n));
-	isuppz = (int *) R_alloc(2*n, sizeof(int));
-	/* ask for optimal size of work arrays */
-	lwork = -1; liwork = -1;
-#ifdef HAVE_LAPACK
-	F77_CALL(dsyevr)(jobv, range, uplo, &n, rx, &n,
-			 &vl, &vu, &il, &iu, &abstol, &m, rvalues,
-			 REAL(z), &n, isuppz,
-			 &tmp, &lwork, &itmp, &liwork, &info);
-#else
-        F77_CALL(rsyevr)(jobv, range, uplo, &n, rx, &n,
-                         &vl, &vu, &il, &iu, &abstol, &m, rvalues,
-                         REAL(z), &n, isuppz,
-                         &tmp, &lwork, &itmp, &liwork, &info);
-#endif
-	if (info != 0)
-	    error("error code %d from Lapack routine dsyevr", info);
-	lwork = (int) tmp;
-	liwork = itmp;
+    range[0] = 'A';
+    if (!ov) PROTECT(z = allocMatrix(REALSXP, n, n));
+    isuppz = (int *) R_alloc(2*n, sizeof(int));
+    /* ask for optimal size of work arrays */
+    lwork = -1; liwork = -1;
+    F77_CALL(dsyevr)(jobv, range, uplo, &n, rx, &n,
+		     &vl, &vu, &il, &iu, &abstol, &m, rvalues,
+		     REAL(z), &n, isuppz,
+		     &tmp, &lwork, &itmp, &liwork, &info);
+    if (info != 0)
+	error(_("error code %d from Lapack routine '%s'"), info, "dsyevr");
+    lwork = (int) tmp;
+    liwork = itmp;
 
-	work = (double *) R_alloc(lwork, sizeof(double));
-	iwork = (int *) R_alloc(liwork, sizeof(int));
-#ifdef HAVE_LAPACK
-	F77_CALL(dsyevr)(jobv, range, uplo, &n, rx, &n,
-			 &vl, &vu, &il, &iu, &abstol, &m, rvalues,
-			 REAL(z), &n, isuppz,
-			 work, &lwork, iwork, &liwork, &info);
-#else
-        F77_CALL(rsyevr)(jobv, range, uplo, &n, rx, &n,
-                         &vl, &vu, &il, &iu, &abstol, &m, rvalues,
-                         REAL(z), &n, isuppz,
-                         work, &lwork, iwork, &liwork, &info);
-#endif
-	if (info != 0)
-	    error("error code %d from Lapack routine dsyevr", info);
-    }
+    work = (double *) R_alloc(lwork, sizeof(double));
+    iwork = (int *) R_alloc(liwork, sizeof(int));
+    F77_CALL(dsyevr)(jobv, range, uplo, &n, rx, &n,
+		     &vl, &vu, &il, &iu, &abstol, &m, rvalues,
+		     REAL(z), &n, isuppz,
+		     work, &lwork, iwork, &liwork, &info);
+    if (info != 0)
+	error(_("error code %d from Lapack routine '%s'"), info, "dsyevr");
 
     if (!ov) {
 	ret = PROTECT(allocVector(VECSXP, 2));
 	nm = PROTECT(allocVector(STRSXP, 2));
 	SET_STRING_ELT(nm, 1, mkChar("vectors"));
-	if(strcmp(meth, "dsyevr")) {
-	    SET_VECTOR_ELT(ret, 1, x);
-	} else {
-	    SET_VECTOR_ELT(ret, 1, z);
-	    UNPROTECT_PTR(z);
-	}
+	SET_VECTOR_ELT(ret, 1, z);
+	UNPROTECT_PTR(z);
     } else {
 	ret = PROTECT(allocVector(VECSXP, 1));
 	nm = PROTECT(allocVector(STRSXP, 1));
@@ -256,13 +211,13 @@ static SEXP modLa_rg(SEXP x, SEXP only_values)
     xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
     n = xdims[0];
     if (n != xdims[1])
-	error("x must be a square numeric matrix");
+	error(_("'x' must be a square numeric matrix"));
 
     xvals = (double *) R_alloc(n * n, sizeof(double));
     /* work on a copy of x */
     Memcpy(xvals, REAL(x), (size_t) (n * n));
     ov = asLogical(only_values);
-    if (ov == NA_LOGICAL) error("invalid `only.values'");
+    if (ov == NA_LOGICAL) error(_("invalid 'only.values'"));
     vectors = !ov;
     jobVL[0] = jobVR[0] = 'N';
     left = right = (double *) 0;
@@ -274,30 +229,24 @@ static SEXP modLa_rg(SEXP x, SEXP only_values)
     wI = (double *) R_alloc(n, sizeof(double));
     /* ask for optimal size of work array */
     lwork = -1;
-#ifdef HAVE_LAPACK
     F77_CALL(dgeev)(jobVL, jobVR, &n, xvals, &n, wR, wI,
 		    left, &n, right, &n, &tmp, &lwork, &info);
-#else
-    F77_CALL(rgeev)(jobVL, jobVR, &n, xvals, &n, wR, wI,
-		    left, &n, right, &n, &tmp, &lwork, &info);
-#endif
     if (info != 0)
-	error("error code %d from Lapack routine dgeev", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "dgeev");
     lwork = (int) tmp;
     work = (double *) R_alloc(lwork, sizeof(double));
-#ifdef HAVE_LAPACK
     F77_CALL(dgeev)(jobVL, jobVR, &n, xvals, &n, wR, wI,
 		    left, &n, right, &n, work, &lwork, &info);
-#else
-    F77_CALL(rgeev)(jobVL, jobVR, &n, xvals, &n, wR, wI,
-		    left, &n, right, &n, work, &lwork, &info);
-#endif
     if (info != 0)
-	error("error code %d from Lapack routine dgeev", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "dgeev");
 
     complexValues = FALSE;
     for (i = 0; i < n; i++)
-	if (wI[i] != 0.0) { complexValues = TRUE; break; }
+	/* This test used to be !=0 for R < 2.3.0.  This is OK for 0+0i */
+	if (fabs(wI[i]) >  10 * R_AccuracyInfo.eps * fabs(wR[i])) {
+	    complexValues = TRUE;
+	    break;
+	}
     ret = PROTECT(allocVector(VECSXP, 2));
     nm = PROTECT(allocVector(STRSXP, 2));
     SET_STRING_ELT(nm, 0, mkChar("values"));
@@ -334,26 +283,26 @@ static SEXP modLa_rg(SEXP x, SEXP only_values)
 
 static SEXP modLa_zgesv(SEXP A, SEXP Bin)
 {
-#ifdef HAVE_DOUBLE_COMPLEX
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
     int n, p, info, *ipiv, *Adims, *Bdims;
     Rcomplex *avals;
     SEXP B;
 
     if (!(isMatrix(A) && isComplex(A)))
-	error("A must be a complex matrix");
+	error(_("'a' must be a complex matrix"));
     if (!(isMatrix(Bin) && isComplex(Bin)))
-	error("B must be a complex matrix");
+	error(_("'b' must be a complex matrix"));
     PROTECT(B = duplicate(Bin));
     Adims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
     Bdims = INTEGER(coerceVector(getAttrib(B, R_DimSymbol), INTSXP));
     n = Adims[0];
-    if(n == 0) error("A is 0-diml");
+    if(n == 0) error(_("'a' is 0-diml"));
     p = Bdims[1];
-    if(p == 0) error("no rhs in B");
+    if(p == 0) error(_("no right-hand side in 'b'"));
     if(Adims[1] != n)
-	error("A (%d x %d) must be square", n, Adims[1]);
+	error(_("'a' (%d x %d) must be square"), n, Adims[1]);
     if(Bdims[0] != n)
-	error("B (%d x %d) must be compatible with A (%d x %d)",
+	error(_("'b' (%d x %d) must be compatible with 'a' (%d x %d)"),
 		Bdims[0], p, n, n);
     ipiv = (int *) R_alloc(n, sizeof(int));
 
@@ -362,27 +311,28 @@ static SEXP modLa_zgesv(SEXP A, SEXP Bin)
     Memcpy(avals, COMPLEX(A), (size_t) (n * n));
     F77_CALL(zgesv)(&n, &p, avals, &n, ipiv, COMPLEX(B), &n, &info);
     if (info < 0)
-	error("argument %d of Lapack routine zgesv had illegal value", -info);
+	error(_("argument %d of Lapack routine %s had invalid value"),
+	      -info, "zgesv");
     if (info > 0)
-	error("Lapack routine zgesv: system is exactly singular");
+	error(("Lapack routine zgesv: system is exactly singular"));
     UNPROTECT(1);
     return B;
 #else
-    error("Fortran complex functions are not available on this platform");
+    error(_("Fortran complex functions are not available on this platform"));
     return R_NilValue; /* -Wall */
 #endif
 }
 
 static SEXP modLa_zgeqp3(SEXP Ain)
 {
-#ifdef HAVE_DOUBLE_COMPLEX
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
     int i, m, n, *Adims, info, lwork;
     Rcomplex *work, tmp;
     double *rwork;
     SEXP val, nm, jpvt, tau, rank, A;
 
     if (!(isMatrix(Ain) && isComplex(Ain)))
-	error("A must be a complex matrix");
+	error(_("'a' must be a complex matrix"));
     PROTECT(A = duplicate(Ain));
     Adims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
     m = Adims[0];
@@ -396,13 +346,13 @@ static SEXP modLa_zgeqp3(SEXP Ain)
     F77_CALL(zgeqp3)(&m, &n, COMPLEX(A), &m, INTEGER(jpvt), COMPLEX(tau),
 		     &tmp, &lwork, rwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine zgeqp3", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zgeqp3");
     lwork = (int) tmp.r;
     work = (Rcomplex *) R_alloc(lwork, sizeof(Rcomplex));
     F77_CALL(zgeqp3)(&m, &n, COMPLEX(A), &m, INTEGER(jpvt), COMPLEX(tau),
 		     work, &lwork, rwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine zgeqp3", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zgeqp3");
     val = PROTECT(allocVector(VECSXP, 4));
     nm = PROTECT(allocVector(STRSXP, 4));
     rank = PROTECT(allocVector(INTSXP, 1));
@@ -419,105 +369,105 @@ static SEXP modLa_zgeqp3(SEXP Ain)
     UNPROTECT(6);
     return val;
 #else
-    error("Fortran complex functions are not available on this platform");
+    error(_("Fortran complex functions are not available on this platform"));
     return R_NilValue; /* -Wall */
 #endif
 }
 
 static SEXP modqr_coef_cmplx(SEXP Q, SEXP Bin)
 {
-#ifdef HAVE_DOUBLE_COMPLEX
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
     int n, nrhs, lwork, info, k, *Bdims, *Qdims;
     SEXP B, qr=VECTOR_ELT(Q, 0), tau=VECTOR_ELT(Q, 2);
     Rcomplex *work, tmp;
 
     k = LENGTH(tau);
     if (!(isMatrix(Bin) && isComplex(Bin)))
-	error("B must be a complex matrix");
+	error(_("'b' must be a complex matrix"));
 
     PROTECT(B = duplicate(Bin));
     Qdims = INTEGER(coerceVector(getAttrib(qr, R_DimSymbol), INTSXP));
     n = Qdims[0];
     Bdims = INTEGER(coerceVector(getAttrib(Bin, R_DimSymbol), INTSXP));
     if(Bdims[0] != n)
-	error("rhs should have %d not %d rows", n, Bdims[0]);
+	error(_("right-hand side should have %d not %d rows"), n, Bdims[0]);
     nrhs = Bdims[1];
     lwork = -1;
     F77_CALL(zunmqr)("L", "C", &n, &nrhs, &k,
 		     COMPLEX(qr), &n, COMPLEX(tau), COMPLEX(B), &n,
 		     &tmp, &lwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine zunmqr", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zunmqr");
     lwork = (int) tmp.r;
     work = (Rcomplex *) R_alloc(lwork, sizeof(Rcomplex));
     F77_CALL(zunmqr)("L", "C", &n, &nrhs, &k,
 		     COMPLEX(qr), &n, COMPLEX(tau), COMPLEX(B), &n,
 		     work, &lwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine zunmqr", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zunmqr");
     F77_CALL(ztrtrs)("U", "N", "N", &k, &nrhs,
 		     COMPLEX(qr), &n, COMPLEX(B), &n, &info);
     if (info != 0)
-	error("error code %d from Lapack routine ztrtrs", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "ztrtrs");
     UNPROTECT(1);
     return B;
 #else
-    error("Fortran complex functions are not available on this platform");
+    error(_("Fortran complex functions are not available on this platform"));
     return R_NilValue; /* -Wall */
 #endif
 }
 
 static SEXP modqr_qy_cmplx(SEXP Q, SEXP Bin, SEXP trans)
 {
-#ifdef HAVE_DOUBLE_COMPLEX
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
     int n, nrhs, lwork, info, k, *Bdims, *Qdims, tr;
     SEXP B, qr=VECTOR_ELT(Q, 0), tau=VECTOR_ELT(Q, 2);
     Rcomplex *work, tmp;
 
     k = LENGTH(tau);
     if (!(isMatrix(Bin) && isComplex(Bin)))
-	error("B must be a complex matrix");
+	error(_("'b' must be a complex matrix"));
     tr = asLogical(trans);
-    if(tr == NA_LOGICAL) error("invalid `trans' parameter");
+    if(tr == NA_LOGICAL) error(_("invalid 'trans' parameter"));
 
     PROTECT(B = duplicate(Bin));
     Qdims = INTEGER(coerceVector(getAttrib(qr, R_DimSymbol), INTSXP));
     n = Qdims[0];
     Bdims = INTEGER(coerceVector(getAttrib(B, R_DimSymbol), INTSXP));
     if(Bdims[0] != n)
-	error("rhs should have %d not %d rows", n, Bdims[0]);
+	error(_("right-hand side should have %d not %d rows"), n, Bdims[0]);
     nrhs = Bdims[1];
     lwork = -1;
     F77_CALL(zunmqr)("L", tr ? "C" : "N", &n, &nrhs, &k,
 		     COMPLEX(qr), &n, COMPLEX(tau), COMPLEX(B), &n,
 		     &tmp, &lwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine zunmqr", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zunmqr");
     lwork = (int) tmp.r;
     work = (Rcomplex *) R_alloc(lwork, sizeof(Rcomplex));
     F77_CALL(zunmqr)("L", tr ? "C" : "N", &n, &nrhs, &k,
 		     COMPLEX(qr), &n, COMPLEX(tau), COMPLEX(B), &n,
 		     work, &lwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine zunmqr", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zunmqr");
     UNPROTECT(1);
     return B;
 #else
-    error("Fortran complex functions are not available on this platform");
+    error(_("Fortran complex functions are not available on this platform"));
     return R_NilValue; /* -Wall */
 #endif
 }
 
 static SEXP modLa_svd_cmplx(SEXP jobu, SEXP jobv, SEXP xin, SEXP s, SEXP u, SEXP v)
 {
-#ifdef HAVE_DOUBLE_COMPLEX
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
     int *xdims, n, p, lwork, info;
     double *rwork;
     Rcomplex *work, tmp;
     SEXP x, val, nm;
 
     if (!(isString(jobu) && isString(jobv)))
-	error("jobu and jobv must be character objects");
+	error(_("'jobu' and 'jobv' must be character strings"));
     PROTECT(x = duplicate(xin));
     xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
     n = xdims[0]; p = xdims[1];
@@ -530,7 +480,7 @@ static SEXP modLa_svd_cmplx(SEXP jobu, SEXP jobv, SEXP xin, SEXP s, SEXP u, SEXP
 		     COMPLEX(v), INTEGER(getAttrib(v, R_DimSymbol)),
 		     &tmp, &lwork, rwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine zgesvd", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zgesvd");
     lwork = (int) tmp.r;
     work = (Rcomplex *) R_alloc(lwork, sizeof(Rcomplex));
     F77_CALL(zgesvd)(CHAR(STRING_ELT(jobu, 0)), CHAR(STRING_ELT(jobv, 0)),
@@ -539,7 +489,7 @@ static SEXP modLa_svd_cmplx(SEXP jobu, SEXP jobv, SEXP xin, SEXP s, SEXP u, SEXP
 		     COMPLEX(v), INTEGER(getAttrib(v, R_DimSymbol)),
 		     work, &lwork, rwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine zgesvd", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zgesvd");
     val = PROTECT(allocVector(VECSXP, 3));
     nm = PROTECT(allocVector(STRSXP, 3));
     SET_STRING_ELT(nm, 0, mkChar("d"));
@@ -552,14 +502,14 @@ static SEXP modLa_svd_cmplx(SEXP jobu, SEXP jobv, SEXP xin, SEXP s, SEXP u, SEXP
     UNPROTECT(3);
     return val;
 #else
-    error("Fortran complex functions are not available on this platform");
+    error(_("Fortran complex functions are not available on this platform"));
     return R_NilValue; /* -Wall */
 #endif
 }
 
 static SEXP modLa_rs_cmplx(SEXP xin, SEXP only_values)
 {
-#ifdef HAVE_DOUBLE_COMPLEX
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
     int *xdims, n, lwork, info, ov;
     char jobv[1], uplo[1];
     SEXP values, ret, nm, x;
@@ -572,9 +522,9 @@ static SEXP modLa_rs_cmplx(SEXP xin, SEXP only_values)
     xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
     n = xdims[0];
     if (n != xdims[1])
-	error("x must be a square numeric matrix");
+	error(_("'x' must be a square numeric matrix"));
     ov = asLogical(only_values);
-    if (ov == NA_LOGICAL) error("invalid `only.values'");
+    if (ov == NA_LOGICAL) error(_("invalid 'only.values'"));
     if (ov) jobv[0] = 'N'; else jobv[0] = 'V';
 
     PROTECT(values = allocVector(REALSXP, n));
@@ -585,13 +535,13 @@ static SEXP modLa_rs_cmplx(SEXP xin, SEXP only_values)
     F77_CALL(zheev)(jobv, uplo, &n, rx, &n, rvalues, &tmp, &lwork, rwork,
 		    &info);
     if (info != 0)
-	error("error code %d from Lapack routine zheev", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zheev");
     lwork = (int) tmp.r;
     work = (Rcomplex *) R_alloc(lwork, sizeof(Rcomplex));
     F77_CALL(zheev)(jobv, uplo, &n, rx, &n, rvalues, work, &lwork, rwork,
 		    &info);
     if (info != 0)
-	error("error code %d from Lapack routine zheev", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zheev");
     if (!ov) {
 	ret = PROTECT(allocVector(VECSXP, 2));
 	nm = PROTECT(allocVector(STRSXP, 2));
@@ -608,14 +558,14 @@ static SEXP modLa_rs_cmplx(SEXP xin, SEXP only_values)
     UNPROTECT(4);
     return ret;
 #else
-    error("Fortran complex functions are not available on this platform");
+    error(_("Fortran complex functions are not available on this platform"));
     return R_NilValue; /* -Wall */
 #endif
 }
 
 static SEXP modLa_rg_cmplx(SEXP x, SEXP only_values)
 {
-#ifdef HAVE_DOUBLE_COMPLEX
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
     int  n, lwork, info, *xdims, ov;
     Rcomplex *work, *left, *right, *xvals, tmp;
     double *rwork;
@@ -625,13 +575,13 @@ static SEXP modLa_rg_cmplx(SEXP x, SEXP only_values)
     xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
     n = xdims[0];
     if (n != xdims[1])
-	error("x must be a square numeric matrix");
+	error(_("'x' must be a square numeric matrix"));
 
     xvals = (Rcomplex *) R_alloc(n * n, sizeof(Rcomplex));
     /* work on a copy of x */
     Memcpy(xvals, COMPLEX(x), (size_t) (n * n));
     ov = asLogical(only_values);
-    if (ov == NA_LOGICAL) error("invalid `only.values'");
+    if (ov == NA_LOGICAL) error(_("invalid 'only.values'"));
     jobVL[0] = jobVR[0] = 'N';
     left = right = (Rcomplex *) 0;
     if (!ov) {
@@ -646,13 +596,13 @@ static SEXP modLa_rg_cmplx(SEXP x, SEXP only_values)
     F77_CALL(zgeev)(jobVL, jobVR, &n, xvals, &n, COMPLEX(values),
 		    left, &n, right, &n, &tmp, &lwork, rwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine zgeev", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zgeev");
     lwork = (int) tmp.r;
     work = (Rcomplex *) R_alloc(lwork, sizeof(Rcomplex));
     F77_CALL(zgeev)(jobVL, jobVR, &n, xvals, &n, COMPLEX(values),
 		    left, &n, right, &n, work, &lwork, rwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine zgeev", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "zgeev");
 
     if(!ov){
 	ret = PROTECT(allocVector(VECSXP, 2));
@@ -669,7 +619,7 @@ static SEXP modLa_rg_cmplx(SEXP x, SEXP only_values)
     UNPROTECT(ov ? 3 : 4);
     return ret;
 #else
-    error("Fortran complex functions are not available on this platform");
+    error(_("Fortran complex functions are not available on this platform"));
     return R_NilValue; /* -Wall */
 #endif
 }
@@ -686,8 +636,8 @@ static SEXP modLa_chol(SEXP A)
 	int n = INTEGER(adims)[1];
 	int i, j;
 
-	if (m != n) error("A must be a square matrix");
-	if (m <= 0) error("A must have dims > 0");
+	if (m != n) error(_("'a' must be a square matrix"));
+	if (m <= 0) error(_("'a' must have dims > 0"));
 	for (j = 0; j < n; j++) {	/* zero the lower triangle */
 	    for (i = j+1; i < n; i++) {
 		REAL(ans)[i + j * n] = 0.;
@@ -697,14 +647,15 @@ static SEXP modLa_chol(SEXP A)
 	F77_CALL(dpotrf)("Upper", &m, REAL(ans), &m, &i);
 	if (i != 0) {
 	    if (i > 0)
-		error("the leading minor of order %d is not positive definite",
+		error(_("the leading minor of order %d is not positive definite"),
 		      i);
-	    error("argument no. %d to Lapack routine dpotrf is illegal", -i);
+	    error(_("argument %d of Lapack routine %s had invalid value"),
+		  -i, "dpotrf");
 	}
 	unprotect(1);
 	return ans;
     }
-    else error("A must be a numeric matrix");
+    else error(_("'a' must be a numeric matrix"));
     return R_NilValue; /* -Wall */
 }
 
@@ -712,7 +663,7 @@ static SEXP modLa_chol2inv(SEXP A, SEXP size)
 {
     int sz = asInteger(size);
     if (sz == NA_INTEGER || sz < 1)
-	error("size argument must be a positive integer");
+	error(_("'size' argument must be a positive integer"));
     if (isMatrix(A)) {
 	SEXP Amat = PROTECT(coerceVector(A, REALSXP));
 	SEXP ans;
@@ -721,8 +672,8 @@ static SEXP modLa_chol2inv(SEXP A, SEXP size)
 	int n = INTEGER(adims)[1];
 	int i, j;
 
-	if (sz > n) error("size cannot exceed ncol(x) = %d", n);
-	if (sz > m) error("size cannot exceed nrow(x) = %d", m);
+	if (sz > n) error(_("'size' cannot exceed ncol(x) = %d"), n);
+	if (sz > m) error(_("'size' cannot exceed nrow(x) = %d"), m);
 	ans = PROTECT(allocMatrix(REALSXP, sz, sz));
 	for (j = 0; j < sz; j++) {
 	    for (i = 0; i <= j; i++)
@@ -731,8 +682,9 @@ static SEXP modLa_chol2inv(SEXP A, SEXP size)
 	F77_CALL(dpotri)("Upper", &sz, REAL(ans), &sz, &i);
 	if (i != 0) {
 	    if (i > 0)
-		error("element (%d, %d) is zero, so the inverse cannot be computed", i, i);
-	    error("argument no. %d to Lapack routine dpotri is illegal", -i);
+		error(_("element (%d, %d) is zero, so the inverse cannot be computed"), i, i);
+	    error(_("argument %d of Lapack routine %s had invalid value"),
+		  -i, "dpotri");
 	}
 	for (j = 0; j < sz; j++) {
 	    for (i = j+1; i < sz; i++)
@@ -741,7 +693,7 @@ static SEXP modLa_chol2inv(SEXP A, SEXP size)
 	unprotect(2);
 	return ans;
     }
-    else error("A must be a numeric matrix");
+    else error(_("'a' must be a numeric matrix"));
     return R_NilValue; /* -Wall */
 }
 
@@ -754,20 +706,20 @@ static SEXP modLa_dgesv(SEXP A, SEXP Bin, SEXP tolin)
     SEXP B;
 
     if (!(isMatrix(A) && isReal(A)))
-	error("A must be a numeric matrix");
+	error(_("'a' must be a numeric matrix"));
     if (!(isMatrix(Bin) && isReal(Bin)))
-	error("B must be a numeric matrix");
+	error(_("'b' must be a numeric matrix"));
     PROTECT(B = duplicate(Bin));
     Adims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
     Bdims = INTEGER(coerceVector(getAttrib(B, R_DimSymbol), INTSXP));
     n = Adims[0];
-    if(n == 0) error("A is 0-diml");
+    if(n == 0) error(_("'a' is 0-diml"));
     p = Bdims[1];
-    if(p == 0) error("no rhs in B");
+    if(p == 0) error(_("no right-hand side in 'b'"));
     if(Adims[1] != n)
-	error("A (%d x %d) must be square", n, Adims[1]);
+	error(_("'a' (%d x %d) must be square"), n, Adims[1]);
     if(Bdims[0] != n)
-	error("B (%d x %d) must be compatible with A (%d x %d)",
+	error(_("'b' (%d x %d) must be compatible with 'a' (%d x %d)"),
 	      Bdims[0], p, n, n);
     ipiv = (int *) R_alloc(n, sizeof(int));
 
@@ -776,14 +728,15 @@ static SEXP modLa_dgesv(SEXP A, SEXP Bin, SEXP tolin)
     Memcpy(avals, REAL(A), (size_t) (n * n));
     F77_CALL(dgesv)(&n, &p, avals, &n, ipiv, REAL(B), &n, &info);
     if (info < 0)
-	error("argument %d of Lapack routine dgesv had illegal value", -info);
+	error(_("argument %d of Lapack routine %s had invalid value"),
+	      -info, "dgesv");
     if (info > 0)
-	error("Lapack routine dgesv: system is exactly singular");
+	error(_("Lapack routine dgesv: system is exactly singular"));
     anorm = F77_CALL(dlange)("1", &n, &n, REAL(A), &n, (double*) NULL);
     work = (double *) R_alloc(4*n, sizeof(double));
     F77_CALL(dgecon)("1", &n, avals, &n, &anorm, &rcond, work, ipiv, &info);
     if (rcond < tol)
-	error("system is computationally singular: reciprocal condition number = %g",
+	error(_("system is computationally singular: reciprocal condition number = %g"),
 	      rcond);
     UNPROTECT(1);
     return B;
@@ -796,7 +749,7 @@ static SEXP modLa_dgeqp3(SEXP Ain)
     SEXP val, nm, jpvt, tau, rank, A;
 
     if (!(isMatrix(Ain) && isReal(Ain)))
-	error("A must be a numeric matrix");
+	error(_("'a' must be a numeric matrix"));
     PROTECT(A = duplicate(Ain));
     Adims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
     m = Adims[0];
@@ -809,13 +762,13 @@ static SEXP modLa_dgeqp3(SEXP Ain)
     F77_CALL(dgeqp3)(&m, &n, REAL(A), &m, INTEGER(jpvt), REAL(tau),
 		     &tmp, &lwork, &info);
     if (info < 0)
-	error("error code %d from Lapack routine dgeqp3", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "dgeqp3");
     lwork = (int) tmp;
     work = (double *) R_alloc(lwork, sizeof(double));
     F77_CALL(dgeqp3)(&m, &n, REAL(A), &m, INTEGER(jpvt), REAL(tau),
 		     work, &lwork, &info);
     if (info < 0)
-	error("error code %d from Lapack routine dgeqp3", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "dgeqp3");
     val = PROTECT(allocVector(VECSXP, 4));
     nm = PROTECT(allocVector(STRSXP, 4));
     rank = PROTECT(allocVector(INTSXP, 1));
@@ -841,32 +794,32 @@ static SEXP modqr_coef_real(SEXP Q, SEXP Bin)
 
     k = LENGTH(tau);
     if (!(isMatrix(Bin) && isReal(Bin)))
-	error("B must be a numeric matrix");
+	error(_("'b' must be a numeric matrix"));
 
     PROTECT(B = duplicate(Bin));
     Qdims = INTEGER(coerceVector(getAttrib(qr, R_DimSymbol), INTSXP));
     n = Qdims[0];
     Bdims = INTEGER(coerceVector(getAttrib(B, R_DimSymbol), INTSXP));
     if(Bdims[0] != n)
-	error("rhs should have %d not %d rows", n, Bdims[0]);
+	error(_("right-hand side should have %d not %d rows"), n, Bdims[0]);
     nrhs = Bdims[1];
     lwork = -1;
     F77_CALL(dormqr)("L", "T", &n, &nrhs, &k,
 		     REAL(qr), &n, REAL(tau), REAL(B), &n,
 		     &tmp, &lwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine dormqr", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "dormqr");
     lwork = (int) tmp;
     work = (double *) R_alloc(lwork, sizeof(double));
     F77_CALL(dormqr)("L", "T", &n, &nrhs, &k,
 		     REAL(qr), &n, REAL(tau), REAL(B), &n,
 		     work, &lwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine dormqr", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "dormqr");
     F77_CALL(dtrtrs)("U", "N", "N", &k, &nrhs,
 		     REAL(qr), &n, REAL(B), &n, &info);
     if (info != 0)
-	error("error code %d from Lapack routine dtrtrs", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "dtrtrs");
     UNPROTECT(1);
     return B;
 }
@@ -879,30 +832,30 @@ static SEXP modqr_qy_real(SEXP Q, SEXP Bin, SEXP trans)
 
     k = LENGTH(tau);
     if (!(isMatrix(Bin) && isReal(Bin)))
-	error("B must be a numeric matrix");
+	error(_("'b' must be a numeric matrix"));
     tr = asLogical(trans);
-    if(tr == NA_LOGICAL) error("invalid `trans' parameter");
+    if(tr == NA_LOGICAL) error(_("invalid 'trans' parameter"));
 
     PROTECT(B = duplicate(Bin));
     Qdims = INTEGER(coerceVector(getAttrib(qr, R_DimSymbol), INTSXP));
     n = Qdims[0];
     Bdims = INTEGER(coerceVector(getAttrib(B, R_DimSymbol), INTSXP));
     if(Bdims[0] != n)
-	error("rhs should have %d not %d rows", n, Bdims[0]);
+	error(_("right-hand side should have %d not %d rows"), n, Bdims[0]);
     nrhs = Bdims[1];
     lwork = -1;
     F77_CALL(dormqr)("L", tr ? "T" : "N", &n, &nrhs, &k,
 		     REAL(qr), &n, REAL(tau), REAL(B), &n,
 		     &tmp, &lwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine dormqr", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "dormqr");
     lwork = (int) tmp;
     work = (double *) R_alloc(lwork, sizeof(double));
     F77_CALL(dormqr)("L", tr ? "T" : "N", &n, &nrhs, &k,
 		     REAL(qr), &n, REAL(tau), REAL(B), &n,
 		     work, &lwork, &info);
     if (info != 0)
-	error("error code %d from Lapack routine dormqr", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "dormqr");
     UNPROTECT(1);
     return B;
 }
@@ -914,19 +867,19 @@ static SEXP moddet_ge_real(SEXP Ain, SEXP logarithm)
     SEXP val, nm, A;
 
     if (!(isMatrix(Ain) && isReal(Ain)))
-	error("A must be a real matrix");
+	error(_("'a' must be a numeric matrix"));
     useLog = asLogical(logarithm);
-    if (useLog == NA_LOGICAL) error("argument logarithm must be logical");
+    if (useLog == NA_LOGICAL) error(_("argument 'logarithm' must be logical"));
     PROTECT(A = duplicate(Ain));
     Adims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
     n = Adims[0];
     if (Adims[1] != n)
-	error("A must be a square matrix");
+	error(_("'a' must be a square matrix"));
     jpvt = (int *) R_alloc(n, sizeof(int));
     F77_CALL(dgetrf)(&n, &n, REAL(A), &n, jpvt, &info);
     sign = 1;
     if (info < 0)
-	error("error code %d from Lapack routine dgetrf", info);
+	error(_("error code %d from Lapack routine '%s'"), info, "dgetrf");
     else if (info > 0) { /* Singular matrix:  U[i,i] (i := info) is 0 */
 	/*warning("Lapack dgetrf(): singular matrix: U[%d,%d]=0", info,info);*/
 	modulus = (useLog ? R_NegInf : 0.);
@@ -966,15 +919,14 @@ static SEXP moddet_ge_real(SEXP Ain, SEXP logarithm)
 
 /* ------------------------------------------------------------ */
 
-
-#include <R_ext/Rlapack.h>
+#include <Rmodules/Rlapack.h>
 #include <R_ext/Rdynload.h>
 
 void
 R_init_lapack(DllInfo *info)
 {
     R_LapackRoutines *tmp;
-    tmp = (R_LapackRoutines*) malloc(sizeof(R_LapackRoutines));
+    tmp = Calloc(1, R_LapackRoutines);
 
     tmp->svd = modLa_svd;
     tmp->rs = modLa_rs;
@@ -995,16 +947,3 @@ R_init_lapack(DllInfo *info)
     tmp->det_ge_real = moddet_ge_real;
     R_setLapackRoutines(tmp);
 }
-
-#ifdef Win32
-#include "psignal.h"
-/* force in malloc & free, so ATLAS gets the right ones */
-/* also force in signal, although what's using that is unclear */
-void lapack_dummy()
-{
-    char *foo;
-    foo = (char *) malloc(1);
-    free(foo);
-    signal(SIGBREAK, NULL);
-}
-#endif
